/*********************************************************************
 * Licensed Materials - PropertyImpl of IBM                              *
 *                                                                   *
 * 5724-B88                                                          *
 *                                                                   *
 *  Copyright IBM Corp.  2001, 2002    All Rights Reserved.         *
 *                                                                   *
 * US Government Users Restricted Rights - Use, duplication or       *
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp. *
 *********************************************************************/

package com.ibm.websphere.query.base;

import java.util.ArrayList;
import java.util.List;

/**
	<P>The Function class represents a function in the query framework. It can be used in a 
	predicate, an order by, and where ever appropriate.
	
	<P>This class contains a name and a vector of elements to describe a typical function. For 
	example, a simple function "upper(name)" is described with an instance of Function class 
	whose name = "upper" and function element = Attribute("name")
 */
public class Function implements IFunction
{
  private String name = "";
  private List functionElements = new ArrayList();
  /**
  	Default contructor.
   */
  public Function()
  {
    super();
  }
  /**
  	Creates a new Function instance with the given name.
   */
  public Function(String newName)
  {
    name = newName;
  }
  /**
  	Creates a new Function instance with the given name, and an array of function elements.
   */
  public Function(String newName, IFunctionElement[] newFunctionElements)
  {
    name = newName;
    setFunctionElements(newFunctionElements);
  }
  /**
  	Creates a new Function instance with the given name, and function element.
   */
  public Function(String newName, IFunctionElement newFunctionElement)
  {
    name = newName;
    setFunctionElement(newFunctionElement);
  }
  /**
  	Adds a function element the function element vector.
   */
  public void addFunctionElement(IFunctionElement functionElement)
  {
    functionElements.add(functionElement);
  }
  /**
  	Builds a query string for the function.
   */
  public String buildString(ISelectQueryCallback builder) throws QueryException
  {
    return builder.buildFunction(this);
  }
  public void buildString(ISelectQueryCallback builder, StringBuffer sb) throws QueryException
  {
    builder.buildFunction(this, sb);
  }
  /**
  	Tests for equality with the given object.
   */
  public boolean equals(Object o)
  {
    boolean eq = true;

    if (o instanceof Function)
    {
      Function f = (Function) o;

      if (name != null)
      {
        eq = name.equals(f.name);
      }
      else
      {
        eq = (f.name == null);
      }

      if (eq)
      {
        eq = functionElements.equals(f.functionElements);
      }

    }
    else
    {
      eq = false;
    }

    return eq;
  }
  /**
  	Returns the function elements as an array.
   */
  public IFunctionElement[] getFunctionElements()
  {
    IFunctionElement fa[] = new IFunctionElement[functionElements.size()];

    for (int i = 0; i < functionElements.size(); i++)
    {
      fa[i] = (IFunctionElement) (functionElements.get(i));
    }

    return fa;
  }
  /**
  	Returns the function name.
   */
  public String getName()
  {
    return name;
  }
  /**
  	Computes a hash code for the function.
   */
  public int hashCode()
  {
    int hashCode = 0;

    if (name != null)
      hashCode = name.hashCode();

    for (int i = 0; i < functionElements.size(); i++)
    {
      hashCode = hashCode ^ functionElements.get(i).hashCode();
    }

    return hashCode;
  }
  /**
  	Sets the function element of the condition.
   */
  public void setFunctionElement(IFunctionElement newFunctionElement)
  {
    functionElements.clear();
    functionElements.add(newFunctionElement);
  }
  /**
  	Sets the function elements of the condition.
   */
  public void setFunctionElements(IFunctionElement[] newFunctionElements)
  {
    functionElements.clear();

    for (int i = 0; i < newFunctionElements.length; i++)
    {
      functionElements.add(newFunctionElements[i]);
    }
  }
  /**
  	Sets the function name.
   */
  public void setName(java.lang.String newName)
  {
    name = newName;
  }
  /**
   * @see ILeftOperand#getDataType()
   */
  public int getDataType()
  {
    int dataType = Constants.DATATYPE_UNKNOWN;
    if (functionElements == null || functionElements.isEmpty())
    {
    }
    else
    {
      dataType = ((IFunctionElement) functionElements.get(0)).getDataType();
    }
    return dataType;
  }

}
