/*********************************************************************
 * Licensed Materials - PropertyImpl of IBM                              *
 *                                                                   *
 * 5724-B88                                                          *
 *                                                                   *
 *  Copyright IBM Corp.  2001, 2002    All Rights Reserved.         *
 *                                                                   *
 * US Government Users Restricted Rights - Use, duplication or       *
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp. *
 *********************************************************************/

package com.ibm.websphere.query.base;


/**
	<P>The Value class represents a general value in the query framework.

	<P>This class contains a value string and data type to describe a value in a query. For example,
	an integer value "30" will be constructed as value string = "30", and data type = 
	Constants.DATATYPE_INTEGER, while a string value "Smith" being value string = "30", and data
	type = Constants.DATATYPE_CHAR
	
	<P>This class also provides the ability to specify a property
	name if this value is associated with a property.  This is useful
	when the data type cannot be determined by the query builder.
 */
public class Value implements IValue
{
  private String valueString = "";
  private int dataType = Constants.DATATYPE_UNKNOWN;
  private String propertyName = null;
  /**
  	Creates a new Value instance, default constructor.
   */
  public Value()
  {
    super();
  }
  /**
  	Creates a new Value instance with the given value string.
   */
  public Value(String newValueString)
  {
    valueString = newValueString;
  }
  /**
  	Creates a new Value instance with the given value string and data type.
   */
  public Value(String newValueString, int newDataType)
  {
    valueString = newValueString;
    dataType = newDataType;
  }
  /**
  	Creates a new Value instance with the given value string and property name.
   */
  public Value(String newValueString, String newPropertyName)
  {
    valueString = newValueString;
    propertyName = newPropertyName;
  }

  /**
  	Creates a new Value instance as a copy of the original value.
   */
  public Value(Value value)
  {
     valueString = value.valueString;
     dataType = value.dataType;
     propertyName = value.propertyName;
  }
  /**
  	Builds a query string for the value.
   */
  public String buildString(ISelectQueryCallback builder) throws QueryException
  {
    return builder.buildValue(this);
  }
  public void buildString(ISelectQueryCallback builder, StringBuffer sb) throws QueryException
  {
    builder.buildValue(this, sb);
  }
  /**
  	Tests for equality with the given object.
   */
  public boolean equals(Object o)
  {
    boolean eq = true;

    if (o instanceof Value)
    {
      Value v = (Value) o;

      if (valueString != null)
      {
        eq = valueString.equals(v.valueString);
      }
      else
      {
        eq = (v.valueString == null);
      }

      if (eq)
      {
        eq = (dataType == v.dataType);
      }

    }
    else
    {
      eq = false;
    }

    return eq;
  }
  /**
  	Returns the data type.
   */
  public int getDataType()
  {
    return dataType;
  }
  /**
  	Returns the value string.
   */
  public String getValueString()
  {
    return valueString;
  }

  /**
  	Returns the property name
   */
  public String getPropertyName()
  {
    return propertyName;
  }

  /**
  	Computes a hash code for the value.
   */
  public int hashCode()
  {
    int hashCode = 0;

    if (valueString != null)
      hashCode = valueString.hashCode();

    hashCode = hashCode * 37 + dataType;

    return hashCode;
  }
  /**
  	Sets the data type of the value.
   */
  public void setDataType(int newDataType)
  {
    dataType = newDataType;
  }
  /**
  	Sets the value string of the value.
   */
  public void setValueString(java.lang.String newValueString)
  {
    valueString = newValueString;
  }

  /**
  	Sets the name of the property this value is associated with.
  	It is not required that all query builders designate a property
  	name for each value, though it is recommended if the data type
  	of the value cannot be determined.
   */
  public void setPropertyName(java.lang.String newPropertyName)
  {
    propertyName = newPropertyName;
  }
}
