/*********************************************************************
 * Licensed Materials - Property of IBM                              *
 *                                                                   *
 * 5724-B88                                                          *
 *                                                                   *
 *  Copyright IBM Corp.  2001, 2002    All Rights Reserved.         *
 *                                                                   *
 * US Government Users Restricted Rights - Use, duplication or       *
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp. *
 *********************************************************************/

package com.ibm.websphere.query.callbacks;

import java.io.UnsupportedEncodingException;
import java.util.HashMap;
import java.util.Map;
import java.util.StringTokenizer;

import com.ibm.icm.util.UtilFactory;
import com.ibm.icm.util.XMLEncoderDecoder;
import com.ibm.icm.util.XMLEncodingDecodingException;

public class CmPathUtil {
	
	public static final String UTF8 = "UTF-8"; //$NON-NLS-1$
	private static final String EMPTY_STRING = ""; //$NON-NLS-1$
	
	public static final char CM_PATH_CHAR = '/'; //$NON-NLS-1$
	public static final String CM_ROOT_PATH = "/"; //$NON-NLS-1$
	public static final char CM_NAMESPACE_CHAR = ':'; //$NON-NLS-1$
	public static final String CM_PATH_SEPARATOR = "/"; //$NON-NLS-1$
	public static final char[] DEFAULT_INVALID_NAME_CHARS = 
	{'\\', '\\', ':', '|', '@', '*', '?', '<', '>', '%', '\'', '\"', '/', '#', '+', '&', '[', ']'};

	
	private CmPathUtil() {
	}

	public static String xmlEncodePath(String path) {
		String encodedPath = null;
		XMLEncoderDecoder encoder = UtilFactory.getXMLEncoderDecoder();
		try {
			encodedPath = encoder.encodeName(path,XMLEncoderDecoder.TYPE_NAME_JCR,XMLEncoderDecoder.TYPE_NAME_XML);
		//	encodedPath = encodedPath.replaceAll("\\.","_x002e_"); //$NON-NLS-1$ //$NON-NLS-2$
			encodedPath = encodedPath.replaceAll(Character.toString((char)160),"_x0020_"); //$NON-NLS-1$ //$NON-NLS-2$
		} catch ( XMLEncodingDecodingException e ) {
			encodedPath = path;
		}
		return encodedPath;
		
	}

	/**
	 * Returns everything before the last index of ".".  Does nothing if
	 * "." not found in stringToTrim
	 * @param stringToTrim the string to trim
	 * @return the trimmed string
	 */
	public static String trimExtension(String stringToTrim) {
		String trimmed = null;
		if (stringToTrim != null) {
			int index = stringToTrim.lastIndexOf('.');
			if ( index != -1) {
				trimmed = stringToTrim.substring(0, index);
			}
		}
		return trimmed;
	}

	/**
	 * Returns the extension of stringWithExtension or "" if string has no extension
	 * or null if stringWithExtension is null.
	 * @param stringWithExtension the string to trim (must have at least four characters)
	 * @return the extension of stringWithExtension or "" if string has no extension or null if stringWithExtension is null
	 */
	public static String getExtension(String stringWithExtension) {
		String extension = null;
		if (stringWithExtension != null) {
			extension = EMPTY_STRING;
			if (stringWithExtension.length() > 4) {
				int extensionIndex = stringWithExtension.lastIndexOf("."); //$NON-NLS-1$
				if (extensionIndex >= 0)
					extension = stringWithExtension.substring(extensionIndex, stringWithExtension.length());
			}
		}
		return extension;
	}

	/**
	 * Returns everything after the last index of ".".  Does nothing if
	 * "." not found in stringToTrim
	 * @param className the string to trim
	 * @return the trimmed string
	 */
	public static String trimPackage(String className) {
		return trimToLastIndexOf(className, "."); //$NON-NLS-1$
	}

	/**
	 * Returns everything after the last index of "/".  Does nothing if
	 * "/" not found in stringToTrim
	 * @param stringToTrim the string to trim
	 * @return the trimmed string
	 */
	public static String trimPath(String stringToTrim) {
		return trimToLastIndexOf(stringToTrim, "/"); //$NON-NLS-1$
	}

	/**
	 * Returns everything before the last index of "/".  Does nothing if
	 * "/" not found in stringToTrim
	 * @param stringToTrim the string to trim
	 * @return the trimmed string
	 */
	public static String trimName(String stringToTrim) {
		String trimmed = null;
		if (stringToTrim != null) {
			int dotIndex = stringToTrim.lastIndexOf(CM_PATH_SEPARATOR);
			if (dotIndex >= 0) {
				trimmed = stringToTrim.substring(0, dotIndex);
			}
		}
		return trimmed;
	}
	
	
	/**
	 * Returns the last path segments.  For example:
	 * 
	 *     "/a/b/c"        -> "c"
	 *     "//a///b/c"     -> "c"
	 *     "a/b/c"         -> "c"
	 *     "/a"            -> "a"
	 *     "a"             -> "a"
	 *     ""              -> ""
	 * 
	 * @param path
	 * @return the last path segment
	 */
	public static String getLastSegment(String path) {
		int index = path.lastIndexOf(CM_PATH_SEPARATOR);
		if (index > -1) {
			if (index > path.length()-1)
				path = path.substring(index+1);
			else
				path = "";
		}
		return path;
	}
	
	/**
	 * Returns the absolute form of the first segment of the path
	 * 
	 *     "/a/b/c"        -> "/a"
	 *     "//a///b/"      -> "/a"
	 *     "a/b/c"         -> "/a"
	 *     "/a"            -> "/a"
	 *     "a"             -> "/a"
	 *     "/"             -> "/"
	 *     ""              -> "/"
	 *     null            -> "/"
	 * 
	 * @param path
	 * @return
	 */
	public static String getFirstSegment(String path) {
		String absPath = getAbsPath(path);
		if ( !CM_ROOT_PATH.equals(absPath) ) {
			int secondSlash = absPath.indexOf(CM_PATH_SEPARATOR);
			secondSlash = absPath.indexOf(CM_PATH_SEPARATOR,secondSlash+1);
			if ( secondSlash != -1 )
				absPath = absPath.substring(0,secondSlash);
		}
		return absPath;
	}
	
	/**
	 * Returns the path segments in order.  For example:
	 * 
	 *     /a/b/c      -> [a,b,c]
	 *     //a///b/c   -> [a,b,c]
	 *     a/b/c       -> [a,b,c]
	 * 
	 * @param path
	 * @return the path segments
	 */
	public static String[] getSegments(String path) {
		path = normalizePath(path);
		StringTokenizer tokenizer = new StringTokenizer(path, CM_PATH_SEPARATOR, false);
		int tokenCount = tokenizer.countTokens();
		String[] segments = new String[tokenCount];
		for (int i=0; i < tokenCount; i++) {
			segments[i] = tokenizer.nextToken();
		}
		return segments;
	}
	
	/**
	 * Returns a new path from the segments.  For example:
	 * 
	 *     [a,b,c]     -> a/b/c
	 *     [/a,b,/c]   -> /a/b//c
	 *     [a,//b,//c] -> a///b///c
	 * 
	 * @param segments
	 * @return the path formed by the segments
	 */
	public static String combineSegments(String[] segments) {
		StringBuffer buffer = new StringBuffer();
		if (segments.length > 0) {
			buffer.append(segments[0]);
		}
		for (int i=1; i<segments.length; i++) {
			buffer.append(CM_PATH_SEPARATOR);
			buffer.append(segments[i]);
		}
		return buffer.toString();
	}

	/**
	 * Returns the parent of this path.  For example:
	 * 
	 * 		/abc/c 		-> /abc
	 * 		/abc		-> /
	 * 		/			-> null
	 * 		/abc/		-> /abc
	 * 
	 * @return The parent path or null if stringToTrim was the root path
	 */
	public static String getParentPath(String stringToTrim) {
		if (stringToTrim != null) {
			int dotIndex = stringToTrim.lastIndexOf(CM_PATH_SEPARATOR);
			if (dotIndex > 0)
				return stringToTrim.substring(0, dotIndex);
			else if (dotIndex == 0) {
				if (stringToTrim.length() > 1)
					return CM_PATH_SEPARATOR;
				else
					return null;
			}
		}
		return null;
	}

	/**
	 * Returns everything after the last index of separator.  Does nothing if
	 * separator not found in stringToTrim
	 * @param stringToTrim the string to trim
	 * @return the trimmed string
	 */
	public static String trimToLastIndexOf(String stringToTrim, String separator) {
		String trimmed = null;
		if (stringToTrim != null) {
			int dotIndex = stringToTrim.lastIndexOf(separator);
			if (dotIndex >= 0) {
				trimmed = stringToTrim.substring(dotIndex + 1);
			}
		}
		return trimmed;
	}

	/**
	 * Removes starting and ending slashes from string, and converts any backslashes into slashes
	 * @param stringToTrim
	 * @return
	 */
	public static String trimSlashes(String stringToTrim) {
		if ( stringToTrim == null )
			return EMPTY_STRING;
		String trimmed = stringToTrim.replace('\\', '/').trim();
		int begin = 0;
		int end = trimmed.length();
		for (; begin < end; begin++)
			if (trimmed.charAt(begin) != '/')
				break;
		for (; end > begin; end--)
			if (trimmed.charAt(end - 1) != '/')
				break;
		trimmed = trimmed.substring(begin, end).trim();
		return trimmed;
	}

	/**
	 * Returns package of className
	 * @param className the string to get the package of
	 * @return the package of className
	 */
	public static String getPackage(String className) {
		return getToLastIndexOf(className, '.', '.');
	}

	public static String combinePaths(String rootPath, String childPath) {
		if (childPath == null)
			return rootPath;
		if (rootPath == null)
			return childPath;

		StringBuffer absPath = new StringBuffer();
		absPath.append(CM_PATH_SEPARATOR);
		absPath.append(rootPath);
		absPath.append(CM_PATH_SEPARATOR);
		absPath.append(childPath);
		absPath = normalizePath(absPath);
		return absPath.toString();
	}
	
	public static String getAbsPath(String path) {
		String absPath = path;
		if (path == null || path.trim().length() == 0) {
			absPath = CM_PATH_SEPARATOR;
		} else if (path.charAt(0) != CM_PATH_CHAR) {
			absPath = CM_PATH_CHAR + path;
		}
		return absPath;
	}
	
	public static StringBuffer normalizePath(StringBuffer absPath) {
		int i = 0;
		boolean fExtra = false;
		while (i < absPath.length()) {
			if (absPath.charAt(i) == CM_PATH_CHAR) {
				if (fExtra) {
					absPath.deleteCharAt(i);
					continue;
				} else
					fExtra = true;
			} else
				fExtra = false;
			i++;
		}
		return absPath;
	}
	
	public static String normalizePath(String absPath) {
		return normalizePath(new StringBuffer(absPath)).toString();
	}

	/**
	 * Returns true if <b>otherPath</b> is a direct child of <b>parentPath</b>
	 * @param parentPath
	 * @param otherPath
	 * @return
	 */
	public static boolean pathIsChild(String parentPath, String otherPath) {
		String parentPathWithSlash = ensureTrailingSlash(parentPath);
		if (otherPath != null && otherPath.startsWith(parentPathWithSlash))
			if (otherPath.lastIndexOf(CM_PATH_SEPARATOR) < parentPathWithSlash.length())
				return true;
		return false;
	}

	/**
	 * Returns true if <b>otherPath</b> is a descendant of <b>parentPath</b>
	 * @param parentPath
	 * @param otherPath
	 * @return
	 */
	public static boolean pathIsDescendant(String parentPath, String otherPath) {
		String parentPathWithSlash = ensureTrailingSlash(parentPath);
		if (otherPath != null && otherPath.startsWith(parentPathWithSlash))
			return true;
		return false;
	}

	/**
	 * Returns true if <b>path</b> is a parent of <b>child</b>
	 * @param path
	 * @param child
	 * @return
	 */
	public static boolean pathIsParent(String path, String child) {
		if (child == null || path == null)
			return false;

		String newPath = ensureTrailingSlash(path);
		return (child.startsWith(newPath) && (child.length() > newPath.length()));
	}

	/**
	 * Returns true if <b>path</b> is a parent of <b>child</b> or they are equivalent
	 * @param path
	 * @param child
	 * @return
	 */
	public static boolean pathIsParentOrIndentical(String path, String child) {
		if (child == null || path == null)
			return false;
		if (child.equals(path))
			return true;

		String newPath = ensureTrailingSlash(path);
		return (child.startsWith(newPath) && (child.length() > newPath.length()));
	}

	public static String ensureLeadingSlash(String rootPath) {
		if ( rootPath == null )
			return CM_ROOT_PATH;
		else if (!rootPath.startsWith(CM_PATH_SEPARATOR))
			return CM_PATH_SEPARATOR + rootPath;
		else
			return rootPath;
	}

	public static String ensureTrailingSlash(String rootPath) {
		if ( rootPath == null )
			return CM_ROOT_PATH;
		else if (!rootPath.endsWith(CM_PATH_SEPARATOR))
			return rootPath + CM_PATH_SEPARATOR;
		else
			return rootPath;
	}

	public static String getNamespace(String namespacedItem) {
		if ( namespacedItem == null )
			return null;
		else {
			int iChar = namespacedItem.indexOf(CM_NAMESPACE_CHAR);
			if ( iChar == -1 )
				return null;
			else
				return namespacedItem.substring(0,iChar);
		}
	}

	public static String getGeneralType(String namespacedItem) {
		if ( namespacedItem == null )
			return null;
		else {
			int iChar = namespacedItem.indexOf(CM_NAMESPACE_CHAR);
			if ( iChar == -1 )
				return null;
			else
				return namespacedItem.substring(iChar+1);
		}
	}

	/**
	 * Returns to last separator (inclusive) taking beginning and ending separators as parameters
	 * Strips any beginning separator
	 * @param stringWithPath the string to return the path of
	 * @param oldPathSeparator the separator of stringWithPath
	 * @param newPathSeparator the separator of the result 
	 * @return to last separator (inclusive) of stringWithPath or null if stringWithPath is null
	 */
	public static String getToLastIndexOf(String stringWithPath, char oldPathSeparator, char newPathSeparator) {
		String path = null;
		if (stringWithPath != null) {
			path = EMPTY_STRING;
			String pathSeparator = new String(new char[] { newPathSeparator });
			int dotIndex = stringWithPath.lastIndexOf(oldPathSeparator);
			if (dotIndex >= 0) {
				path = (stringWithPath.substring(0, dotIndex + 1)).replace(oldPathSeparator, newPathSeparator);
				if (path.startsWith(pathSeparator)) {
					path = path.substring(1);
				}
			}
		}
		return path;
	}

	/**
	 * Validates the name of an Authortime resource such as a Campaign, Rule,
	 * RuleMapping and EmailPromotion..
	 * @param resName name of the resource
	 * @param sb(output) string buffer that contains the list of invalid chars
	 * @return boolean true if name is valid, false otherwise
	 */
	public static boolean isValidResourceName(String resName, StringBuffer sb) {
		return isValidResourceName(resName,sb,DEFAULT_INVALID_NAME_CHARS);
	}
	
	/**
	 * Validates the name of an Authortime resource such as a Campaign, Rule,
	 * RuleMapping and EmailPromotion..
	 * @param resName name of the resource
	 * @param sb(output) string buffer that contains the list of invalid chars
	 * @return boolean true if name is valid, false otherwise
	 */
	public static boolean isValidResourceName(String resName, StringBuffer sb, char[] badChars) {
		/*
		 *  Note, this method is duplicated in com.ibm.dm.pzn.ui.util.PathUtil 
		 * 	All changes should be mirrored there.
		 */

		for (int i = 0; i < resName.length(); i++) {
			for (int j = 0; j < badChars.length; j++) {
				if (resName.charAt(i) == badChars[j]) {
					sb.append(badChars[j]);
					break;
				}
			}
		}
		return (sb.length() > 0) ? false : true;
	}

	/**
	 * 
	 * @param s1
	 * @return
	 */
	public static String htmlEscape(String s1) {
		if ( s1 == null )
			return null;
		StringBuffer buf = new StringBuffer();
		int i;
		for (i = 0; i < s1.length(); ++i) {
			char ch = s1.charAt(i);
			String entity = (String) s_i2e.get(new Integer((int) ch));
			if (entity == null) {
				if (((int) ch) > 128) {
					buf.append("&#" + ((int) ch) + ";"); //$NON-NLS-1$ //$NON-NLS-2$
				} else {
					buf.append(ch);
				}
			} else {
				buf.append("&" + entity + ";"); //$NON-NLS-1$ //$NON-NLS-2$
			}
		}
		return buf.toString();
	}

	/**
	 * 
	 * @param s1
	 * @return
	 */
	public static String htmlUnescape(String s1) {
		if ( s1 == null )
			return null;
		StringBuffer buf = new StringBuffer();
		int i;
		for (i = 0; i < s1.length(); ++i) {
			char ch = s1.charAt(i);
			if (ch == '&') {
				int semi = s1.indexOf(';', i + 1);
				if (semi == -1) {
					buf.append(ch);
					continue;
				}
				String entity = s1.substring(i + 1, semi);
				Integer iso;
				if (entity.charAt(0) == '#') {
					iso = new Integer(entity.substring(1));
				} else {
					iso = (Integer) s_e2i.get(entity);
				}
				if (iso == null) {
					buf.append("&" + entity + ";"); //$NON-NLS-1$ //$NON-NLS-2$
				} else {
					buf.append((char) (iso.intValue()));
				}
				i = semi;
			} else {
				buf.append(ch);
			}
		}
		return buf.toString();
	}

	static Object[][] entities = {
		// {"#39", new Integer(39)},       // ' - apostrophe
		{ "quot", new Integer(34)}, // " - double-quote //$NON-NLS-1$
		{
			"amp", new Integer(38) //$NON-NLS-1$

		
		}, // & - ampersand
		{
			"lt", new Integer(60) //$NON-NLS-1$

		
		}, // < - less-than
		{
			"gt", new Integer(62) //$NON-NLS-1$

		
		}, // > - greater-than
		{
			"nbsp", new Integer(160) //$NON-NLS-1$

		
		}, // non-breaking space
		{
			"copy", new Integer(169) //$NON-NLS-1$

		
		}, //  - copyright
		{
			"reg", new Integer(174) //$NON-NLS-1$

		
		}, //  - registered trademark
		{
			"Agrave", new Integer(192) //$NON-NLS-1$

		
		}, //  - uppercase A, grave accent
		{
			"Aacute", new Integer(193) //$NON-NLS-1$

		
		}, //  - uppercase A, acute accent
		{
			"Acirc", new Integer(194) //$NON-NLS-1$

		
		}, //  - uppercase A, circumflex accent
		{
			"Atilde", new Integer(195) //$NON-NLS-1$

		
		}, //  - uppercase A, tilde
		{
			"Auml", new Integer(196) //$NON-NLS-1$

		
		}, //  - uppercase A, umlaut
		{
			"Aring", new Integer(197) //$NON-NLS-1$

		
		}, //  - uppercase A, ring
		{
			"AElig", new Integer(198) //$NON-NLS-1$

		
		}, //  - uppercase AE
		{
			"Ccedil", new Integer(199) //$NON-NLS-1$

		
		}, //  - uppercase C, cedilla
		{
			"Egrave", new Integer(200) //$NON-NLS-1$

		
		}, //  - uppercase E, grave accent
		{
			"Eacute", new Integer(201) //$NON-NLS-1$

		
		}, //  - uppercase E, acute accent
		{
			"Ecirc", new Integer(202) //$NON-NLS-1$

		
		}, //  - uppercase E, circumflex accent
		{
			"Euml", new Integer(203) //$NON-NLS-1$

		
		}, //  - uppercase E, umlaut
		{
			"Igrave", new Integer(204) //$NON-NLS-1$

		
		}, //  - uppercase I, grave accent
		{
			"Iacute", new Integer(205) //$NON-NLS-1$

		
		}, //  - uppercase I, acute accent
		{
			"Icirc", new Integer(206) //$NON-NLS-1$

		
		}, //  - uppercase I, circumflex accent
		{
			"Iuml", new Integer(207) //$NON-NLS-1$

		
		}, //  - uppercase I, umlaut
		{
			"ETH", new Integer(208) //$NON-NLS-1$

		
		}, //  - uppercase Eth, Icelandic
		{
			"Ntilde", new Integer(209) //$NON-NLS-1$

		
		}, //  - uppercase N, tilde
		{
			"Ograve", new Integer(210) //$NON-NLS-1$

		
		}, //  - uppercase O, grave accent
		{
			"Oacute", new Integer(211) //$NON-NLS-1$

		
		}, //  - uppercase O, acute accent
		{
			"Ocirc", new Integer(212) //$NON-NLS-1$

		
		}, //  - uppercase O, circumflex accent
		{
			"Otilde", new Integer(213) //$NON-NLS-1$

		
		}, //  - uppercase O, tilde
		{
			"Ouml", new Integer(214) //$NON-NLS-1$

		
		}, //  - uppercase O, umlaut
		{
			"Oslash", new Integer(216) //$NON-NLS-1$

		
		}, //  - uppercase O, slash
		{
			"Ugrave", new Integer(217) //$NON-NLS-1$

		
		}, //  - uppercase U, grave accent
		{
			"Uacute", new Integer(218) //$NON-NLS-1$

		
		}, //  - uppercase U, acute accent
		{
			"Ucirc", new Integer(219) //$NON-NLS-1$

		
		}, //  - uppercase U, circumflex accent
		{
			"Uuml", new Integer(220) //$NON-NLS-1$

		
		}, //  - uppercase U, umlaut
		{
			"Yacute", new Integer(221) //$NON-NLS-1$

		
		}, //  - uppercase Y, acute accent
		{
			"THORN", new Integer(222) //$NON-NLS-1$

		
		}, //  - uppercase THORN, Icelandic
		{
			"szlig", new Integer(223) //$NON-NLS-1$

		
		}, //  - lowercase sharps, German
		{
			"agrave", new Integer(224) //$NON-NLS-1$

		
		}, //  - lowercase a, grave accent
		{
			"aacute", new Integer(225) //$NON-NLS-1$

		
		}, //  - lowercase a, acute accent
		{
			"acirc", new Integer(226) //$NON-NLS-1$

		
		}, //  - lowercase a, circumflex accent
		{
			"atilde", new Integer(227) //$NON-NLS-1$

		
		}, //  - lowercase a, tilde
		{
			"auml", new Integer(228) //$NON-NLS-1$

		
		}, //  - lowercase a, umlaut
		{
			"aring", new Integer(229) //$NON-NLS-1$

		
		}, //  - lowercase a, ring
		{
			"aelig", new Integer(230) //$NON-NLS-1$

		
		}, //  - lowercase ae
		{
			"ccedil", new Integer(231) //$NON-NLS-1$

		
		}, //  - lowercase c, cedilla
		{
			"egrave", new Integer(232) //$NON-NLS-1$

		
		}, //  - lowercase e, grave accent
		{
			"eacute", new Integer(233) //$NON-NLS-1$

		
		}, //  - lowercase e, acute accent
		{
			"ecirc", new Integer(234) //$NON-NLS-1$

		
		}, //  - lowercase e, circumflex accent
		{
			"euml", new Integer(235) //$NON-NLS-1$

		
		}, //  - lowercase e, umlaut
		{
			"igrave", new Integer(236) //$NON-NLS-1$

		
		}, //  - lowercase i, grave accent
		{
			"iacute", new Integer(237) //$NON-NLS-1$

		
		}, //  - lowercase i, acute accent
		{
			"icirc", new Integer(238) //$NON-NLS-1$

		
		}, //  - lowercase i, circumflex accent
		{
			"iuml", new Integer(239) //$NON-NLS-1$

		
		}, //  - lowercase i, umlaut
		{
			"igrave", new Integer(236) //$NON-NLS-1$

		
		}, //  - lowercase i, grave accent
		{
			"iacute", new Integer(237) //$NON-NLS-1$

		
		}, //  - lowercase i, acute accent
		{
			"icirc", new Integer(238) //$NON-NLS-1$

		
		}, //  - lowercase i, circumflex accent
		{
			"iuml", new Integer(239) //$NON-NLS-1$

		
		}, //  - lowercase i, umlaut
		{
			"eth", new Integer(240) //$NON-NLS-1$

		
		}, //  - lowercase eth, Icelandic
		{
			"ntilde", new Integer(241) //$NON-NLS-1$

		
		}, //  - lowercase n, tilde
		{
			"ograve", new Integer(242) //$NON-NLS-1$

		
		}, //  - lowercase o, grave accent
		{
			"oacute", new Integer(243) //$NON-NLS-1$

		
		}, //  - lowercase o, acute accent
		{
			"ocirc", new Integer(244) //$NON-NLS-1$

		
		}, //  - lowercase o, circumflex accent
		{
			"otilde", new Integer(245) //$NON-NLS-1$

		
		}, //  - lowercase o, tilde
		{
			"ouml", new Integer(246) //$NON-NLS-1$

		
		}, //  - lowercase o, umlaut
		{
			"oslash", new Integer(248) //$NON-NLS-1$

		
		}, //  - lowercase o, slash
		{
			"ugrave", new Integer(249) //$NON-NLS-1$

		
		}, //  - lowercase u, grave accent
		{
			"uacute", new Integer(250) //$NON-NLS-1$

		
		}, //  - lowercase u, acute accent
		{
			"ucirc", new Integer(251) //$NON-NLS-1$

		
		}, //  - lowercase u, circumflex accent
		{
			"uuml", new Integer(252) //$NON-NLS-1$

		
		}, //  - lowercase u, umlaut
		{
			"yacute", new Integer(253) //$NON-NLS-1$

		
		}, //  - lowercase y, acute accent
		{
			"thorn", new Integer(254) //$NON-NLS-1$

		
		}, //  - lowercase thorn, Icelandic
		{
			"yuml", new Integer(255) //$NON-NLS-1$

		
		}, //  - lowercase y, umlaut
		{
			"euro", new Integer(8364) //$NON-NLS-1$

		
		}, // Euro symbol
	};
	static Map s_e2i = new HashMap();
	static Map s_i2e = new HashMap();
	static {
		for (int i = 0; i < entities.length; ++i) {
			s_e2i.put(entities[i][0], entities[i][1]);
			s_i2e.put(entities[i][1], entities[i][0]);
		}
	}

	public static String convertEncoding(String s, String enc1, String enc2) {
		try {
			String encoded = (s != null) ? new String(s.getBytes(enc1),enc2) : null; //$NON-NLS-1$
			return encoded;
		} catch ( UnsupportedEncodingException e ) {
			throw new IllegalArgumentException(e.getLocalizedMessage()); 
		}
	}

	public static String convertEncodingToUTF8(String s, String enc1) {
		try {
			String encoded = (s != null) ? new String(s.getBytes(enc1),UTF8) : null; //$NON-NLS-1$
			return encoded;
		} catch ( UnsupportedEncodingException e ) {
			throw new IllegalArgumentException(e.getLocalizedMessage()); 
		}
	}

}
