package com.ibm.websphere.query.callbacks;

import java.util.Hashtable;
import java.util.Map;

import com.ibm.websphere.query.base.ArithmeticCondition;
import com.ibm.websphere.query.base.Attribute;
import com.ibm.websphere.query.base.Constants;
import com.ibm.websphere.query.base.Function;
import com.ibm.websphere.query.base.IArithmeticCondition;
import com.ibm.websphere.query.base.IAttribute;
import com.ibm.websphere.query.base.ICondition;
import com.ibm.websphere.query.base.IFunction;
import com.ibm.websphere.query.base.ILeftOperand;
import com.ibm.websphere.query.base.ILimit;
import com.ibm.websphere.query.base.IOperator;
import com.ibm.websphere.query.base.IOrderBy;
import com.ibm.websphere.query.base.IOrderByExpression;
import com.ibm.websphere.query.base.IPredicate;
import com.ibm.websphere.query.base.IQuery;
import com.ibm.websphere.query.base.IRightOperand;
import com.ibm.websphere.query.base.ISelectQuery;
import com.ibm.websphere.query.base.IValue;
import com.ibm.websphere.query.base.PredicateBase;
import com.ibm.websphere.query.base.QueryException;
import com.ibm.websphere.query.base.Value;

/**
 * Licensed Materials - Property of IBM. (C) COPYRIGHT International Business Machines Corp. 2002.
 */

/**
	<P>The EipSelectQueryCallback class is a query builder callback for EIP. It provides 
	attributes mapping and query syntax conversion for developers to build meaningful EIP
	query strings.
		
	<P>It implments the ISelectQueryCallback interface so it can be used whenever an ISelectQueryCallback
	is needed.
	
	<P>This class has a hash table that contains EIP attributes mapping. When building an 
	attribute string, the hash table is looked up to perform an attribute conversion. Multi-value
	conditions are not supported in EIP.
 */
public class EipSelectQueryCallback extends SelectQueryCallbackAdapter implements com.ibm.websphere.query.base.ISelectQueryCallback {
    public static final String EIP_QUERY_SPACE = " ";
    public static final String EIP_QUERY_STRINGSEP = "'";
    public static final String EIP_QUERY_LEFTPAREN = "(";
    public static final String EIP_QUERY_RIGHTPAREN = ")";
    public static final String EIP_QUERY_COMMA = ",";

    public static final String EIP_QUERY_WHEREHEAD = "COND=";
    public static final String EIP_QUERY_WHERETAIL = "";
    public static final String EIP_QUERY_PREDICATEHEAD = "(";
    public static final String EIP_QUERY_PREDICATETAIL = ")";
    public static final String EIP_QUERY_CONDITIONHEAD = "(";
    public static final String EIP_QUERY_CONDITIONTAIL = ")";

    public static final String EIP_QUERY_AND = "AND";
    public static final String EIP_QUERY_OR = "OR";
    public static final String EIP_QUERY_EQ = "==";
    public static final String EIP_QUERY_NEQ = "<>";
    public static final String EIP_QUERY_LEQ = "<=";
    public static final String EIP_QUERY_LT = "<";
    public static final String EIP_QUERY_GEQ = ">=";
    public static final String EIP_QUERY_GT = ">";
    public static final String EIP_QUERY_BTW = "BETWEEN";
    public static final String EIP_QUERY_IN = "IN";
    public static final String EIP_QUERY_LIKE = "LIKE";
    public static final String EIP_QUERY_NOT_LIKE = "NOT LIKE";
    public static final String EIP_QUERY_NULL = "null";
    public static final String EIP_QUERY_WILDCARD = "%";

    private Map nameMapping = null;
    private Map typeMapping = null;
    /**
    	Creates a new EipSelectQueryCallback instance, default constructor.
     */
    public EipSelectQueryCallback() {
        super();
    }
    /**
    	Creates a new EipSelectQueryCallback instance with the given attribute mapping.
     */
    public EipSelectQueryCallback(Hashtable newNameMapping) {
        super();
        nameMapping = newNameMapping;
    }
    /**
    	Creates a new EipSelectQueryCallback instance with the given attribute and type mappings.
     */
    public EipSelectQueryCallback(Map newNameMapping, Map newTypeMapping) {
        super();
        nameMapping = newNameMapping;
        typeMapping = newTypeMapping;
    }
    /**
    	<P>Returns the EIP federated attribute string for the given attribute. 
     */
    public String buildAttribute(IAttribute attribute) throws QueryException {
        String s = attribute.getName();

        if ((nameMapping != null) && (nameMapping.containsKey(s)))
            s = (String) nameMapping.get(s);

        return EIP_QUERY_STRINGSEP + s + EIP_QUERY_STRINGSEP;
    }
    /**
    	<P>Returns the EIP Federated condition string for the given condition. 
    	
    	<P>This method defines string generation for conditions with all possible operators supported
    	in EIP federated query language. Note that the multi-value operators are not supported in EIP.
     */
    public String buildCondition(ICondition condition) throws QueryException {
        String s = EIP_QUERY_CONDITIONHEAD;

        IOperator o = condition.getOperator();
        ILeftOperand l = condition.getLeftOperand();
        IRightOperand[] r = condition.getRightOperands();

        boolean isRightOperandsNull = false;
        if (r.length == 0 || r[0] == null) {
            isRightOperandsNull = true;
        }
        else if (r[0] instanceof Value) {
            isRightOperandsNull = ((Value) r[0]).getValueString() == null;
        }

        String operatorName = o.getName();

        if (operatorName.equals(Constants.OPERATOR_GT) || operatorName.equals(Constants.OPERATOR_GEQ) || operatorName.equals(Constants.OPERATOR_LT) || operatorName.equals(Constants.OPERATOR_LEQ)) {
            s = s + l.buildString(this);
            s = s + o.buildString(this);
            s = s + r[0].buildString(this);
        }
        else if (operatorName.equals(Constants.OPERATOR_EQ) || operatorName.equals(Constants.OPERATOR_NEQ)) {
            if (isRightOperandsNull) {
                s = s + l.buildString(this);
                s = s + o.buildString(this);
                s = s + EIP_QUERY_NULL;
            }
            else {
                s = s + l.buildString(this);
                s = s + o.buildString(this);
                s = s + r[0].buildString(this);
            }
        }
        else if (operatorName.equals(Constants.OPERATOR_BTW)) {
            s = s + l.buildString(this);
            s = s + EIP_QUERY_SPACE + o.buildString(this) + EIP_QUERY_SPACE;
            s = s + r[0].buildString(this);
            s = s + EIP_QUERY_SPACE;
            s = s + r[1].buildString(this);
        }
        else if (operatorName.equals(Constants.OPERATOR_IN)) {
            if (isRightOperandsNull) {
                s = s + EIP_QUERY_LEFTPAREN + l.buildString(this) + EIP_QUERY_EQ + EIP_QUERY_NULL + EIP_QUERY_RIGHTPAREN;
                s = s + EIP_QUERY_SPACE + EIP_QUERY_AND + EIP_QUERY_SPACE;
                s = s + EIP_QUERY_LEFTPAREN + l.buildString(this) + EIP_QUERY_NEQ + EIP_QUERY_NULL + EIP_QUERY_RIGHTPAREN;
            }
            else {
                s = s + l.buildString(this);
                s = s + EIP_QUERY_SPACE + o.buildString(this) + EIP_QUERY_SPACE + EIP_QUERY_LEFTPAREN;
                for (int i = 0; i < r.length; i++) {
                    if (i > 0)
                        s = s + EIP_QUERY_COMMA;
                    s = s + r[i].buildString(this);
                }
                s = s + EIP_QUERY_RIGHTPAREN;
            }
        }
        else if (operatorName.equals(Constants.OPERATOR_LIKE) || operatorName.equals(Constants.OPERATOR_NOT_LIKE)) {
            if ((l instanceof Attribute) && (r[0] instanceof Value)) {
                Value v = (Value) r[0];
                int dt = v.getDataType();

                if ((dt == Constants.DATATYPE_CHAR) || (dt == Constants.DATATYPE_LONGVARCHAR) || (dt == Constants.DATATYPE_VARCHAR)) {
                    String vs = v.getValueString();

                    if (vs.indexOf(EIP_QUERY_WILDCARD) == -1) {
                        vs = EIP_QUERY_WILDCARD + vs + EIP_QUERY_WILDCARD;
                    }
                    v.setValueString(vs);

                    s = s + l.buildString(this);
                    s = s + EIP_QUERY_SPACE + o.buildString(this) + EIP_QUERY_SPACE;
                    s = s + r[0].buildString(this);
                }
                else {
                    throw new QueryException("Invalid data type for operator '" + operatorName + "'.", 1);
                }
            }
            else {
                throw new QueryException("Operator '" + operatorName + "' not supported in this context.", 1);
            }
        }
        else {
            throw new QueryException("Operator '" + operatorName + "' not supported.", 1);
        }

        s = s + EIP_QUERY_CONDITIONTAIL;
        return s;
    }
    /**
    	<P>Throws an exception because EIP federated query does not support functions.
     */
    public String buildFunction(IFunction function) throws QueryException {
        throw new QueryException("Function '" + function.getName() + "' not supported.", 2);
    }
    /**
    	<P>Returns the EIP federated operator string for the given operator. 
    
    	<P>This method defines string generation for all possible operators supported in EIP federated
    	query language.
     */
    public String buildOperator(IOperator operator) throws QueryException {
        String s = "";

        String operatorName = operator.getName();

        if (operatorName.equals(Constants.OPERATOR_AND)) {
            s = EIP_QUERY_AND;
        }
        else if (operatorName.equals(Constants.OPERATOR_OR)) {
            s = EIP_QUERY_OR;
        }
        else if (operatorName.equals(Constants.OPERATOR_EQ)) {
            s = EIP_QUERY_EQ;
        }
        else if (operatorName.equals(Constants.OPERATOR_NEQ)) {
            s = EIP_QUERY_NEQ;
        }
        else if (operatorName.equals(Constants.OPERATOR_GT)) {
            s = EIP_QUERY_GT;
        }
        else if (operatorName.equals(Constants.OPERATOR_GEQ)) {
            s = EIP_QUERY_GEQ;
        }
        else if (operatorName.equals(Constants.OPERATOR_LT)) {
            s = EIP_QUERY_LT;
        }
        else if (operatorName.equals(Constants.OPERATOR_LEQ)) {
            s = EIP_QUERY_LEQ;
        }
        else if (operatorName.equals(Constants.OPERATOR_BTW)) {
            s = EIP_QUERY_BTW;
        }
        else if (operatorName.equals(Constants.OPERATOR_IN)) {
            s = EIP_QUERY_IN;
        }
        else if (operatorName.equals(Constants.OPERATOR_LIKE)) {
            s = EIP_QUERY_LIKE;
        }
        else if (operatorName.equals(Constants.OPERATOR_NOT_LIKE)) {
            s = EIP_QUERY_NOT_LIKE;
        }
        else {
            throw new QueryException("Operator '" + operatorName + "' not supported.", 1);
        }

        return s;
    }
    /**
    	<P>Throws an exception because EIP federated query does not support order by.
     */
    public String buildOrderBy(IOrderBy orderBy) throws QueryException {
        throw new QueryException("Order by not supported.", 4);
    }
    /**
    	<P>Throws an exception because EIP federated query does not support order by.
     */
    public String buildOrderBy(ISelectQuery selectQuery) throws QueryException {
        throw new QueryException("Order by not supported.", 4);
    }
    /**
    	<P>Throws an exception because EIP federated query does not support order by.
     */
    public String buildOrderByExpression(IOrderByExpression orderByExpression) throws QueryException {
        throw new QueryException("Order by not supported.", 4);
    }
    /**
    	<P>Returns the EIP federated predicate string for the given predicate. 
    
    	<P>This method uses the predicate bases in the predicate to generate the predicate string in
    	EIP federated query language.
     */
    public String buildPredicate(IPredicate predicate) throws QueryException {
        String s = EIP_QUERY_PREDICATEHEAD;

        IOperator o = predicate.getOperator();
        PredicateBase[] pb = predicate.getPredicateBases();

        for (int i = 0; i < pb.length; i++) {
            if (i > 0)
                s = s + EIP_QUERY_SPACE + o.buildString(this) + EIP_QUERY_SPACE;

            s = s + pb[i].buildString(this);
        }
        s = s + EIP_QUERY_PREDICATETAIL;

        return s;
    }
    /**
    	<P>Returns the EIP federated predicate string for the given query. 
    
    	<P>This method uses the predicate in the query to generate the predicate string in EIP
    	federated query language.
     */
    public String buildPredicate(IQuery query) throws QueryException {
        String s = "";
        PredicateBase pb = query.getPredicate();

        if (pb != null) {
            s = EIP_QUERY_WHEREHEAD + pb.buildString(this);
        }
        return s;
    }
    /**
    	<P>Returns the EIP federated query string for the given query. 
    
    	<P>This method calls buildPredicate() to generate the query string in EIP federate query
    	language.
     */
    public String buildQuery(IQuery query) throws QueryException {
        return buildPredicate(query);
    }
    /**
    	<P>Returns the EIP federated select query string for the given select query. 
    
    	<P>This method calls buildPredicate()to generate the select query string in EIP federated
    	query language.
     */
    public String buildSelectQuery(ISelectQuery selectQuery) throws QueryException {
        return buildPredicate(selectQuery);
    }
    /**
    	<P>Returns the EIP federated value string for the given value.
    
    	<P>This method generates the value string in EIP federated query language based on the 
    	data type.
     */
    public String buildValue(IValue value) throws QueryException {
        String s = buildValueString(value.getValueString());

        int dataType = value.getDataType();
        if (typeMapping != null && dataType == Constants.DATATYPE_UNKNOWN) {
            String propertyName = value.getPropertyName();
            if (propertyName != null) {
                Object o = typeMapping.get(propertyName);
                if (o != null && o instanceof Integer) {
                    dataType = ((Integer) o).intValue();
                }
            }
        }

        switch (dataType) {
            case Constants.DATATYPE_BIGINT :
            case Constants.DATATYPE_DECIMAL :
            case Constants.DATATYPE_DOUBLE :
            case Constants.DATATYPE_FLOAT :
            case Constants.DATATYPE_TINYINT :
            case Constants.DATATYPE_NUMERIC :
            case Constants.DATATYPE_INTEGER :
            case Constants.DATATYPE_SMALLINT :
            case Constants.DATATYPE_REAL :
            case Constants.DATATYPE_UNKNOWN :
            case Constants.DATATYPE_NULL :
                break;
            case Constants.DATATYPE_CHAR :
            case Constants.DATATYPE_LONGVARCHAR :
            case Constants.DATATYPE_VARCHAR :
            case Constants.DATATYPE_DATE :
            case Constants.DATATYPE_TIME :
            case Constants.DATATYPE_TIMESTAMP :
                s = EIP_QUERY_STRINGSEP + s + EIP_QUERY_STRINGSEP;
                break;
            default :
                throw new QueryException("Data type not supported.", 3);
        }

        return s;
    }
    /**
    	<P>Returns the EIP federated query safe value string for the given string.
     */
    protected String buildValueString(String valueString) throws QueryException {
        String s = "";
        int i = 0;
        int j = valueString.indexOf(EIP_QUERY_STRINGSEP, i);
        for (;;) {
            if (j == -1) {
                s = s + valueString.substring(i);
                break;
            }
            s = s + valueString.substring(i, j) + EIP_QUERY_STRINGSEP + EIP_QUERY_STRINGSEP;
            i = j + 1;
            j = valueString.indexOf(EIP_QUERY_STRINGSEP, i);
        }
        return s;
    }
    /**
    	Returns the attribute mapping hash table of the builder callback.
     */
    public java.util.Map getNameMapping() {
        return nameMapping;
    }
    /**
    	Sets the attribute mapping hash table of the builder callback.
     */
    public void setNameMapping(java.util.Hashtable newNameMapping) {
        nameMapping = newNameMapping;
    }

    /**
     * @see ISelectQueryCallback#buildArithmeticCondition(IArithmeticCondition)
     */
    public String buildArithmeticCondition(IArithmeticCondition arithmeticCondition) throws QueryException {
        String s = EIP_QUERY_SPACE;
        String arithmeticOperation = arithmeticCondition.getArithmeticOperation();
        ILeftOperand[] arithmeticOperands = arithmeticCondition.getArithmeticConditionElements();
        for (int i = 0; i < arithmeticOperands.length; i++) {
            if (i > 0) {
                s = s + EIP_QUERY_SPACE + arithmeticOperation + EIP_QUERY_SPACE;
            }
            if (arithmeticOperands[i] instanceof Attribute) {
                s = s + ((Attribute) arithmeticOperands[i]).buildString(this);
            }
            else if (arithmeticOperands[i] instanceof Function) {
                s = s + ((Function) arithmeticOperands[i]).buildString(this);
            }
            else if (arithmeticOperands[i] instanceof Value) {
                s = s + ((Value) arithmeticOperands[i]).buildString(this);
            }
            else if (arithmeticOperands[i] instanceof ArithmeticCondition) {
                s = s + EIP_QUERY_LEFTPAREN + ((ArithmeticCondition) arithmeticOperands[i]).buildString(this) + EIP_QUERY_RIGHTPAREN;
            }
        }
        s = s + EIP_QUERY_SPACE;
        return s;
    }

    /**
     * @see ISelectQueryCallback#buildLimit(ILimit)
     */
    public String buildLimit(ILimit limit) throws QueryException {
        //not sure if supported
        return "";
    }

    /**
     * @see ISelectQueryCallback#buildLimit(ISelectQuery)
     */
    public String buildLimit(ISelectQuery selectQuery) throws QueryException {
        //not sure if supported
        return "";
    }

}