package com.ibm.websphere.query.callbacks;

import java.util.Map;

import com.ibm.websphere.query.base.Constants;
import com.ibm.websphere.query.base.IFrom;
import com.ibm.websphere.query.base.ILimit;
import com.ibm.websphere.query.base.ISelectQuery;
import com.ibm.websphere.query.base.ITable;
import com.ibm.websphere.query.base.IValue;
import com.ibm.websphere.query.base.QueryException;

/**
 * Licensed Materials - PropertyImpl of IBM. (C) COPYRIGHT International Business Machines Corp. 2002.
 */

/**
 * <P>The OracleSqlSelectQueryCallback class is a query builder callback that
 * adds the functionality of querying by Date objects on Oracle, and adding limit
 * to Oracle queries. 
 */
public class OracleSqlSelectQueryCallback extends SqlSelectQueryCallback
{

  public static final String ORACLE_SQL_QUERY_LIMITHEAD = "rownum" + SQL_QUERY_LEQ;

  /**
   * Creates a new OracleSqlSelectQueryCallback instance with the given attribute and type mappings.
   */
  public OracleSqlSelectQueryCallback(Map nameMapping, Map typeMapping)
  {
    super(nameMapping, typeMapping);
  }
  /**
  	<P>Returns the Oracle SQL value string for the given value.
  
  	<P>This method generates the value string in SQL based on the data type.
   */
  public void buildValue(IValue value, StringBuffer sb, Map metadata) throws QueryException
  {
    String s = value.getValueString();
    if (s.indexOf("'") != -1)
    {
      s = makeQuerySafeValue(s);
    }

    int dataType = value.getDataType();
    Map typeMapping = getTypeMapping(metadata);
    if (typeMapping != null && dataType == Constants.DATATYPE_UNKNOWN)
    {
      String propertyName = value.getPropertyName();
      if (propertyName != null)
      {
        Object o = typeMapping.get(propertyName);
        if (o != null && o instanceof Integer)
        {
          dataType = ((Integer) o).intValue();
        }
      }
    }

    switch (dataType)
    {
      case Constants.DATATYPE_BIGINT :
      case Constants.DATATYPE_DECIMAL :
      case Constants.DATATYPE_DOUBLE :
      case Constants.DATATYPE_FLOAT :
      case Constants.DATATYPE_TINYINT :
      case Constants.DATATYPE_NUMERIC :
      case Constants.DATATYPE_INTEGER :
      case Constants.DATATYPE_SMALLINT :
      case Constants.DATATYPE_REAL :
      case Constants.DATATYPE_UNKNOWN :
      case Constants.DATATYPE_NULL :
        break;
      case Constants.DATATYPE_CHAR :
      case Constants.DATATYPE_LONGVARCHAR :
      case Constants.DATATYPE_VARCHAR :
        s = SQL_QUERY_STRINGSEP + s + SQL_QUERY_STRINGSEP;
        break;
      case Constants.DATATYPE_DATE :
        s = "to_date('" + s + "','yyyy-mm-dd')";
        break;
      case Constants.DATATYPE_TIME :
        s = "to_date('" + s + "','hh24:mi:ss')";
        break;
      case Constants.DATATYPE_TIMESTAMP :
        if (s.indexOf(".") > 0)
        {
          s = "to_date('" + s.substring(0, s.indexOf(".")) + "','yyyy-mm-dd hh24:mi:ss')";
        }
        else
        {
          s = "to_date('" + s + "','yyyy-mm-dd hh24:mi:ss')";
        }
        break;
      default :
        throw new QueryException("Data type not supported.", 3);
    }

    sb.append(s);
  }

  /**
   * @see com.ibm.websphere.query.base.ISelectQueryCallback#buildLimit(ISelectQuery)
   */
  public void buildLimit(ISelectQuery selectQuery, StringBuffer sb, Map metadata) throws QueryException
  {
    ILimit limit = selectQuery.getLimit();
    if (limit != null)
    {
      sb.append(ORACLE_SQL_QUERY_LIMITHEAD);
      limit.buildString(this, sb);
    }
  }

  /**
   * @see com.ibm.websphere.query.base.ISelectQueryCallback#buildSelectQuery(ISelectQuery)
   */
  public void buildSelectQuery(ISelectQuery selectQuery, StringBuffer sb, Map metadata) throws QueryException
  {
    IFrom from = selectQuery.getFrom();
    if (from != null)
    {
      ITable table = from.getTable();
      metadata.put(TABLE_NAME, table.getName());
    }

    buildPredicate(selectQuery, sb);
    sb.append(SQL_QUERY_SPACE);
    buildOrderBy(selectQuery, sb);
    if (selectQuery.getLimit() != null)
    {
      sb.append(SQL_QUERY_SPACEAND);
      buildLimit(selectQuery, sb, metadata);
    }

    StringBuffer sb2 = new StringBuffer();
    buildSelect(selectQuery, sb2);
    buildFrom(selectQuery, sb2);
    sb.insert(0, sb2.toString());
  }

}
