package com.ibm.websphere.query.callbacks;

import java.util.HashMap;
import java.util.HashSet;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;

import com.ibm.websphere.query.base.ArithmeticCondition;
import com.ibm.websphere.query.base.Attribute;
import com.ibm.websphere.query.base.Constants;
import com.ibm.websphere.query.base.Function;
import com.ibm.websphere.query.base.IArithmeticCondition;
import com.ibm.websphere.query.base.IAttribute;
import com.ibm.websphere.query.base.ICondition;
import com.ibm.websphere.query.base.IDeleteQuery;
import com.ibm.websphere.query.base.IFrom;
import com.ibm.websphere.query.base.IFunction;
import com.ibm.websphere.query.base.IFunctionElement;
import com.ibm.websphere.query.base.IJoin;
import com.ibm.websphere.query.base.ILeftOperand;
import com.ibm.websphere.query.base.ILimit;
import com.ibm.websphere.query.base.IOperator;
import com.ibm.websphere.query.base.IOrderBy;
import com.ibm.websphere.query.base.IOrderByElement;
import com.ibm.websphere.query.base.IOrderByExpression;
import com.ibm.websphere.query.base.IPredicate;
import com.ibm.websphere.query.base.IQuery;
import com.ibm.websphere.query.base.IRightOperand;
import com.ibm.websphere.query.base.ISelect;
import com.ibm.websphere.query.base.ISelectQuery;
import com.ibm.websphere.query.base.ISelectQueryCallback;
import com.ibm.websphere.query.base.ITable;
import com.ibm.websphere.query.base.IValue;
import com.ibm.websphere.query.base.PredicateBase;
import com.ibm.websphere.query.base.QueryException;
import com.ibm.websphere.query.base.Value;

/**
 * Licensed Materials - PropertyImpl of IBM. (C) COPYRIGHT International Business Machines Corp. 2002.
 */

/**
	<P>The SqlSelectQueryCallback class is a query builder callback for SQL. It provides
	attributes mapping and query syntax conversion for developers to build meaningful SQL
	query strings.

	<P>It implments the ISelectQueryCallback interface so it can be used whenever an ISelectQueryCallback
	is needed.

	<P>This class has a hash table that contains SQL attributes mapping. When building an
	attribute string, the hash table is looked up to perform an attribute conversion.It is also
	used to convert multi-value conditions such as MVALL into sub query strings.
 */
public class SqlSelectQueryCallback implements ISelectQueryCallback
{
  public static final String SQL_QUERY_SPACE = " ";
  public static final String SQL_QUERY_STRINGSEP = "'";
  public static final String SQL_QUERY_LEFTPAREN = "(";
  public static final String SQL_QUERY_RIGHTPAREN = ")";
  public static final String SQL_QUERY_COMMA = ",";

  public static final String SQL_QUERY_WHERETAIL = "";
  public static final String SQL_QUERY_CONDITIONHEAD = "(";
  public static final String SQL_QUERY_CONDITIONTAIL = ")";
  public static final String SQL_QUERY_WILDCARD = "%";
  public static final String SQL_QUERY_DOT = ".";
  public static final String SQL_QUERY_PREDICATEHEAD = "(";
  public static final String SQL_QUERY_PREDICATETAIL = ")";

  public static final String SQL_QUERY_AND = "AND ";
  public static final String SQL_QUERY_SPACEAND = SQL_QUERY_SPACE + SQL_QUERY_AND;
  public static final String SQL_QUERY_OR = "OR ";
  public static final String SQL_QUERY_SPACEOR = SQL_QUERY_SPACE + SQL_QUERY_OR;
  public static final String SQL_QUERY_EQ = "=";
  public static final String SQL_QUERY_NEQ = "<>";
  public static final String SQL_QUERY_LEQ = "<=";
  public static final String SQL_QUERY_LT = "<";
  public static final String SQL_QUERY_GEQ = ">=";
  public static final String SQL_QUERY_GT = ">";
  public static final String SQL_QUERY_BTW = "BETWEEN";
  public static final String SQL_QUERY_IN = "IN";
  public static final String SQL_QUERY_LIKE = "LIKE";
  public static final String SQL_QUERY_NOT_LIKE = "NOT LIKE";
  public static final String SQL_QUERY_IS = "IS";
  public static final String SQL_QUERY_ISN = "IS NOT";
  public static final String SQL_QUERY_NULL = "NULL";
  public static final String SQL_QUERY_ISNULL = " IS NULL";
  public static final String SQL_QUERY_ISNNULL = " IS NOT NULL";

  public static final String SQL_QUERY_ARITHMETIC_ADD = "+";
  public static final String SQL_QUERY_ARITHMETIC_SUBTRACT = "-";
  public static final String SQL_QUERY_ARITHMETIC_MULTIPLY = "*";
  public static final String SQL_QUERY_ARITHMETIC_DIVIDE = "/";

  public static final String SQL_QUERY_UPPER = "UPPER";
  public static final String SQL_QUERY_LOWER = "LOWER";

  public static final String SQL_QUERY_SELECT = "SELECT ";
  public static final String SQL_QUERY_SELECT_ALL = "SELECT * ";
  public static final String SQL_QUERY_DELETE = "DELETE ";
  public static final String SQL_QUERY_FROM = "FROM ";
  public static final String SQL_QUERY_WHERE = "WHERE ";

  public static final String SQL_QUERY_ORDERBYHEAD = "ORDER BY ";
  public static final String SQL_QUERY_ORDERBYTAIL = "";
  public static final String SQL_QUERY_ORDERBYEXPSEP = ", ";
  public static final String SQL_QUERY_ORDERBYASIS = "";
  public static final String SQL_QUERY_ORDERBYRANDOM = "";

  public static final String SQL_QUERY_ORDERBYASC = "ASC";
  public static final String SQL_QUERY_ORDERBYDES = "DESC";

  public static final String SQL_QUERY_MVANY = "IN";
  public static final String SQL_QUERY_MVALL = "<MVALL>";

  public static final Map OPERATOR_MAP = new HashMap();
  static {
    OPERATOR_MAP.put(Constants.OPERATOR_AND, SQL_QUERY_AND);
    OPERATOR_MAP.put(Constants.OPERATOR_OR, SQL_QUERY_OR);
    OPERATOR_MAP.put(Constants.OPERATOR_EQ, SQL_QUERY_EQ);
    OPERATOR_MAP.put(Constants.OPERATOR_NEQ, SQL_QUERY_NEQ);
    OPERATOR_MAP.put(Constants.OPERATOR_GT, SQL_QUERY_GT);
    OPERATOR_MAP.put(Constants.OPERATOR_GEQ, SQL_QUERY_GEQ);
    OPERATOR_MAP.put(Constants.OPERATOR_LT, SQL_QUERY_LT);
    OPERATOR_MAP.put(Constants.OPERATOR_LEQ, SQL_QUERY_LEQ);
    OPERATOR_MAP.put(Constants.OPERATOR_BTW, SQL_QUERY_BTW);
    OPERATOR_MAP.put(Constants.OPERATOR_IN, SQL_QUERY_IN);
    OPERATOR_MAP.put(Constants.OPERATOR_LIKE, SQL_QUERY_LIKE);
    OPERATOR_MAP.put(Constants.OPERATOR_NOT_LIKE, SQL_QUERY_NOT_LIKE);
    OPERATOR_MAP.put(Constants.OPERATOR_IS, SQL_QUERY_IS);
    OPERATOR_MAP.put(Constants.OPERATOR_ISN, SQL_QUERY_ISN);
    OPERATOR_MAP.put(Constants.OPERATOR_ASC, SQL_QUERY_ORDERBYASC);
    OPERATOR_MAP.put(Constants.OPERATOR_DES, SQL_QUERY_ORDERBYDES);
  }

  public static final Set MULTIVALUE_SINGLEOPERAND_OPERATOR_SET = new HashSet();
  static {
    MULTIVALUE_SINGLEOPERAND_OPERATOR_SET.add(Constants.OPERATOR_EQ);
    MULTIVALUE_SINGLEOPERAND_OPERATOR_SET.add(Constants.OPERATOR_NEQ);
    MULTIVALUE_SINGLEOPERAND_OPERATOR_SET.add(Constants.OPERATOR_GT);
    MULTIVALUE_SINGLEOPERAND_OPERATOR_SET.add(Constants.OPERATOR_GEQ);
    MULTIVALUE_SINGLEOPERAND_OPERATOR_SET.add(Constants.OPERATOR_LT);
    MULTIVALUE_SINGLEOPERAND_OPERATOR_SET.add(Constants.OPERATOR_LEQ);
    MULTIVALUE_SINGLEOPERAND_OPERATOR_SET.add(Constants.OPERATOR_IS);
    MULTIVALUE_SINGLEOPERAND_OPERATOR_SET.add(Constants.OPERATOR_ISN);
    MULTIVALUE_SINGLEOPERAND_OPERATOR_SET.add(Constants.OPERATOR_LIKE);
    MULTIVALUE_SINGLEOPERAND_OPERATOR_SET.add(Constants.OPERATOR_NOT_LIKE);
  }

  public static final Set SINGLEOPERAND_NONNULL_OPERATOR_SET = new HashSet();
  static {
    SINGLEOPERAND_NONNULL_OPERATOR_SET.add(Constants.OPERATOR_IS);
    SINGLEOPERAND_NONNULL_OPERATOR_SET.add(Constants.OPERATOR_ISN);
    SINGLEOPERAND_NONNULL_OPERATOR_SET.add(Constants.OPERATOR_GT);
    SINGLEOPERAND_NONNULL_OPERATOR_SET.add(Constants.OPERATOR_GEQ);
    SINGLEOPERAND_NONNULL_OPERATOR_SET.add(Constants.OPERATOR_LT);
    SINGLEOPERAND_NONNULL_OPERATOR_SET.add(Constants.OPERATOR_LEQ);
  }

  protected Map nameMapping;
  protected Map typeMapping;
  protected Map objectMapping;

  /**
  	Creates a new SqlSelectQueryCallback instance with the given attribute and
  	type mappings.
   */
  public SqlSelectQueryCallback() {
     this(new HashMap(), new HashMap());
  }

  public SqlSelectQueryCallback(Hashtable nameMapping) {
     this(nameMapping, new HashMap());
  }

  public SqlSelectQueryCallback(Map nameMapping, Map typeMapping)
  {
	this(nameMapping, typeMapping, new HashMap());
  }

  public SqlSelectQueryCallback(Map nameMapping, Map typeMapping, Map objectMapping)
  {
	super();
	this.nameMapping = nameMapping;
	this.typeMapping = typeMapping;
	this.objectMapping = objectMapping;
  }

  /**
  	<P>Returns the SQL attribute string for the given attribute.
  
  	<P>SQL table column name is returned if the attribute mapping contains an entry whose key matches
  	the attribute name.
   */

  public String buildAttribute(IAttribute attribute) throws QueryException
  {
     StringBuffer buf = new StringBuffer();
     buildAttribute(attribute, buf);
     return buf.toString();
  }

  public void buildAttribute(IAttribute attribute, StringBuffer sb) throws QueryException
  {
    String s = attribute.getName();

    if ((nameMapping != null) && (nameMapping.containsKey(s)))
    {
      Object o = nameMapping.get(s);
      if (o instanceof IJoin)
      {
        IJoin join = (IJoin)o;
        IAttribute compareAttribute = join.getCompareAttribute();
        if (compareAttribute == null)
        {
          throw new QueryException("Cannot build join as attribute without compareAttribute", 1);
        }
        else
        {
          compareAttribute.buildString(this, sb);
        }
        s = "";
      }
      else
      {
        String tableName;
        ITable table = attribute.getTable();
        if (table != null)
        {
          tableName = table.getName();
        }
        else
        {
          tableName = getTableName();
        }
		if (tableName == null) {
          s = o.toString();
		} else {
          s = getTableAlias(tableName) + SQL_QUERY_DOT;
          s += o;
        }
      }
    }
    sb.append(s);
   }

  /**
  	<P>Returns the SQL condition string for the given condition.
  
  	<P>This method defines string generation for conditions with all possible operators supported
  	in SQL.	Note that the left operand is checked for multiplicity first because single-value
  	operators have unique meanings for SQL if the left operand is a multi-value attribute.
   */
  public String buildCondition(ICondition condition) throws QueryException
  {
     StringBuffer buf = new StringBuffer();
     buildCondition(condition, buf);
     return buf.toString();
  }

  public void buildCondition(ICondition condition, StringBuffer sb) throws QueryException
  {
    sb.append(SQL_QUERY_CONDITIONHEAD);

    IOperator o = condition.getOperator();
    ILeftOperand l = condition.getLeftOperand();
    IRightOperand[] r = condition.getRightOperands();

    boolean isLeftOperandMVAttribute = false;
    boolean isRightOperandsNull = false;
    if (l instanceof IAttribute)
    {
      isLeftOperandMVAttribute = isMultiValueAttribute((IAttribute) l);
      if (isLeftOperandMVAttribute)
      {
        Set multiValuePropertyNames = (Set) objectMapping.get(MULTIVALUE_PROPERTY_NAMES);
        if (multiValuePropertyNames == null)
        {
          multiValuePropertyNames = new HashSet();
          objectMapping.put(MULTIVALUE_PROPERTY_NAMES, multiValuePropertyNames);
        }
        multiValuePropertyNames.add(((IAttribute) l).getName());
      }
    }
    if (r.length == 0 || r[0] == null)
    {
      isRightOperandsNull = true;
    }
    else if (r[0] instanceof Value)
    {
      isRightOperandsNull = ((Value) r[0]).getValueString() == null;
    }

    String operatorName = o.getName();

    if (SINGLEOPERAND_NONNULL_OPERATOR_SET.contains(operatorName))
    {
      if (isLeftOperandMVAttribute)
      {
        buildSMVCondition(condition, sb);
        //return;
      }
      else
      {
        l.buildString(this, sb);
        sb.append(SQL_QUERY_SPACE);
        o.buildString(this, sb);
        sb.append(SQL_QUERY_SPACE);
        r[0].buildString(this, sb);
      }
    }
    else if (operatorName.equals(Constants.OPERATOR_EQ))
    {
      if (isLeftOperandMVAttribute)
      {
        buildSMVCondition(condition, sb);
        //return;
      }
      else
      {
        if (isRightOperandsNull)
        {
          l.buildString(this, sb);
          sb.append(SQL_QUERY_ISNULL);
        }
        else
        {
          l.buildString(this, sb);
          sb.append(SQL_QUERY_SPACE);
          o.buildString(this, sb);
          sb.append(SQL_QUERY_SPACE);
          r[0].buildString(this, sb);
        }
      }
    }
    else if (operatorName.equals(Constants.OPERATOR_NEQ))
    {
      if (isLeftOperandMVAttribute)
      {
        buildSMVCondition(condition, sb);
        //return;
      }
      else
      {
        if (isRightOperandsNull)
        {
          l.buildString(this, sb);
          sb.append(SQL_QUERY_ISNNULL);
        }
        else
        {
          l.buildString(this, sb);
          sb.append(SQL_QUERY_SPACE);
          o.buildString(this, sb);
          sb.append(SQL_QUERY_SPACE);
          r[0].buildString(this, sb);
        }
      }
    }
    else if (operatorName.equals(Constants.OPERATOR_BTW))
    {
      if (isLeftOperandMVAttribute)
      {
        buildSMVCondition(condition, sb);
        //return;
      }
      else
      {
        l.buildString(this, sb);
        sb.append(SQL_QUERY_SPACE);
        o.buildString(this, sb);
        sb.append(SQL_QUERY_SPACE);
        r[0].buildString(this, sb);
        sb.append(SQL_QUERY_SPACEAND);
        r[1].buildString(this, sb);
      }
    }
    else if (operatorName.equals(Constants.OPERATOR_IN))
    {
      if (isLeftOperandMVAttribute)
      {
        buildSMVCondition(condition, sb);
        //return;
      }
      else
      {
        if (isRightOperandsNull)
        {
          sb.append(SQL_QUERY_LEFTPAREN);
          l.buildString(this, sb);
          sb.append(SQL_QUERY_ISNULL);
          sb.append(SQL_QUERY_RIGHTPAREN);
          sb.append(SQL_QUERY_SPACEAND);
          sb.append(SQL_QUERY_LEFTPAREN);
          l.buildString(this, sb);
          sb.append(SQL_QUERY_ISNNULL);
          sb.append(SQL_QUERY_RIGHTPAREN);
        }
        else
        {
          l.buildString(this, sb);
          sb.append(SQL_QUERY_SPACE);
          o.buildString(this, sb);
          sb.append(SQL_QUERY_SPACE);
          if (r.length == 1 && r[0] instanceof ISelectQuery)
          {
            buildInnerSelect((ISelectQuery) r[0], sb);
          }
          else
          {
            sb.append(SQL_QUERY_SPACE + SQL_QUERY_LEFTPAREN);
            for (int i = 0; i < r.length; i++)
            {
              if (i > 0)
              {
                sb.append(SQL_QUERY_COMMA);
              }
              r[i].buildString(this, sb);
            }
            sb.append(SQL_QUERY_RIGHTPAREN);
          }

        }
      }
    }
    else if (operatorName.equals(Constants.OPERATOR_LIKE) || operatorName.equals(Constants.OPERATOR_NOT_LIKE))
    {
      if (isLeftOperandMVAttribute)
      {
        buildSMVCondition(condition, sb);
        //return;
      }
      else
      {
        if (r[0] instanceof Value)
        {
          Value v = (Value) r[0];
          int dt = v.getDataType();

          if ((dt == Constants.DATATYPE_CHAR) || (dt == Constants.DATATYPE_LONGVARCHAR) || (dt == Constants.DATATYPE_VARCHAR))
          {
            l.buildString(this, sb);
            sb.append(SQL_QUERY_SPACE);
            o.buildString(this, sb);
            sb.append(SQL_QUERY_SPACE);
            int len = sb.length();
            v.buildString(this, sb);
            String vs = v.getValueString();
            if (vs.indexOf(SQL_QUERY_WILDCARD) == -1) {
               sb.insert(len + 1, SQL_QUERY_WILDCARD);
               sb.insert(sb.length() - 1, SQL_QUERY_WILDCARD);
            }
          }
          else
          {
            throw new QueryException("Invalid data type for operator '" + operatorName + "'.", 1);
          }
        }
        else
        {
          throw new QueryException("Operator '" + operatorName + "' not supported in this context.", 1);
        }
      }
    }
    else if (operatorName.equals(Constants.OPERATOR_MVALL)) {
       processMvAll(l, r, sb);
    }
    else if (operatorName.equals(Constants.OPERATOR_MVANY)) {
       processMvAny(l, r, sb);
    }
    else
    {
      throw new QueryException("Operator '" + operatorName + "' not supported.", 1);
    }
    sb.append(SQL_QUERY_CONDITIONTAIL);
  }
  /**
  	<P>Returns the SQL function string for the given function.
  
  	<P>This method defines string generation for all possible functions supported in SQL.
   */
  public String buildFunction(IFunction function) throws QueryException 
  {
     StringBuffer buf = new StringBuffer();
     buildFunction(function, buf);
     return buf.toString();
  }

  public void buildFunction(IFunction function, StringBuffer sb) throws QueryException
  {
    String functionName = function.getName();
    IFunctionElement[] fs = function.getFunctionElements();

    if (functionName.equals(Constants.FUNCTION_UPPER))
    {
      sb.append(SQL_QUERY_UPPER + SQL_QUERY_LEFTPAREN);
      fs[0].buildString(this, sb);
      sb.append(SQL_QUERY_RIGHTPAREN);
    }
    else if (functionName.equals(Constants.FUNCTION_LOWER))
    {
      sb.append(SQL_QUERY_LOWER + SQL_QUERY_LEFTPAREN);
      fs[0].buildString(this, sb);
      sb.append(SQL_QUERY_RIGHTPAREN);
    }
    else
    {
      throw new QueryException("Function '" + functionName + "' not supported.", 2);
    }
  }

  /**
  	<P>Returns the SQL operator string for the given operator.
  
  	<P>This method defines string generation for all possible operators supported in SQL.
   */
  public String buildOperator(IOperator operator) throws QueryException
  {
     StringBuffer buf = new StringBuffer();
     buildOperator(operator, buf);
     return buf.toString();
  }

  public void buildOperator(IOperator operator, StringBuffer sb) throws QueryException
  {
    String operatorName = operator.getName();

    String operatorString = (String) OPERATOR_MAP.get(operatorName);

    if (operatorString != null)
    {
      sb.append(operatorString);
    }
    else
    {
      throw new QueryException("Operator '" + operatorName + "' not supported.", 1);
    }
  }
  /**
  	<P>Returns the SQL order by string for the given order by object.
  
  	<P>This method iterates through the order by expressions in the order by object to generate
  	the order by string in SQL.
   */

  public String buildOrderBy(IOrderBy orderBy) throws QueryException
  {
     StringBuffer buf = new StringBuffer();
     buildOrderBy(orderBy, buf);
     return buf.toString();
  }

  public void buildOrderBy(IOrderBy orderBy, StringBuffer sb) throws QueryException
  {
    IOrderByExpression[] oa = orderBy.getOrderByExpressions();

    for (int i = 0; i < oa.length; i++)
    {
      if (i > 0)
      {
        sb.append(SQL_QUERY_ORDERBYEXPSEP);
      }

      oa[i].buildString(this, sb);
    }
  }
  /**
  	<P>Returns the SQL order by string for the given select query.
  
  	<P>This method uses the order by in the select query to generate the order by string in SQL.
   */
  public String buildOrderBy(ISelectQuery selectQuery) throws QueryException
  {
    StringBuffer buf = new StringBuffer();
    buildOrderBy(selectQuery, buf);
    return buf.toString();
  }

  public void buildOrderBy(ISelectQuery selectQuery, StringBuffer sb) throws QueryException
  {
    IOrderBy o = selectQuery.getOrderBy();

    if (o != null)
    {
      sb.append(SQL_QUERY_ORDERBYHEAD);
      o.buildString(this, sb);
    }
  }
  /**
  	<P>Returns the SQL order by expression string for the given order by expression.
  
  	<P>This method uses the order by element and operator in the order by expression to generate
  	the order by expression string in SQL.
   */
  public String buildOrderByExpression(IOrderByExpression orderByExpression) throws QueryException
  {
     StringBuffer buf = new StringBuffer();
     buildOrderByExpression(orderByExpression, buf);
     return buf.toString();
  }
  public void buildOrderByExpression(IOrderByExpression orderByExpression, StringBuffer sb) throws QueryException
  {
    IOperator o = orderByExpression.getOperator();
    IOrderByElement oe = orderByExpression.getOrderByElement();

    oe.buildString(this, sb);
    sb.append(SQL_QUERY_SPACE);
    o.buildString(this, sb);
  }

  public String buildPredicate(IPredicate predicate) throws QueryException
  {
     StringBuffer buf = new StringBuffer();
     buildPredicate(predicate, buf);
     return buf.toString();
  }
  public void buildPredicate(IPredicate predicate, StringBuffer sb) throws QueryException
  {
    sb.append(SQL_QUERY_PREDICATEHEAD);

    IOperator o = predicate.getOperator();
    PredicateBase[] pb = predicate.getPredicateBases();

    for (int i = 0; i < pb.length; i++)
    {
      if (i > 0)
      {
        sb.append(SQL_QUERY_SPACE);
        o.buildString(this, sb);
        sb.append(SQL_QUERY_SPACE);
      }
      pb[i].buildString(this, sb);
    }
    sb.append(SQL_QUERY_PREDICATETAIL);
  }

  public String buildPredicate(IQuery query) throws QueryException
  {
     StringBuffer buf = new StringBuffer();
     buildPredicate(query, buf);
     return buf.toString();
  }
  public void buildPredicate(IQuery query, StringBuffer sb) throws QueryException
  {
    PredicateBase pb = query.getPredicate();

    if (pb != null)
    {
      sb.append(SQL_QUERY_WHERE);
      pb.buildString(this, sb);
    }
    else
    {
      objectMapping.put(PREDICATE_IS_NULL, Boolean.TRUE);
    }
  }


  /**
     <P>Returns the SQL query string for the given query.

     <P>This method calls buildPredicate() to generate the query string in SQL.
   */
  public String buildQuery(IQuery query) throws QueryException {
     return buildPredicate(query);
  }

  /**
  	<P>Returns the SQL select query string for the given select query.
  
  	<P>This method calls buildPredicate() and buildOrderBy() to generate the select query string
  	in SQL.
   */
  public String buildSelectQuery(ISelectQuery selectQuery) throws QueryException
  {
     StringBuffer buf = new StringBuffer();
     buildSelectQuery(selectQuery, buf);
     return buf.toString();
  }

  public void buildSelectQuery(ISelectQuery selectQuery, StringBuffer sb) throws QueryException
  {
    IFrom from = selectQuery.getFrom();
    if (from != null)
    {
      ITable table = from.getTable();
      objectMapping.put(TABLE_NAME, table.getName());
    }


    buildPredicate(selectQuery, sb);
    sb.append(SQL_QUERY_SPACE);
    buildOrderBy(selectQuery, sb);
    sb.append(SQL_QUERY_SPACE);
    buildLimit(selectQuery, sb);

    if (objectMapping.get(TABLE_NAME) != null) {
      StringBuffer sb2 = new StringBuffer();
      buildSelect(selectQuery, sb2);
      buildFrom(selectQuery, sb2);
      sb.insert(0, sb2.toString());
    }
  }

  public void buildDeleteQuery(IDeleteQuery deleteQuery, StringBuffer sb) throws QueryException
  {
    IFrom from = deleteQuery.getFrom();
    if (from != null)
    {
      ITable table = from.getTable();
      objectMapping.put(TABLE_NAME, table.getName());
    }

    buildPredicate(deleteQuery, sb);

    StringBuffer sb2 = new StringBuffer();
    sb2.append(SQL_QUERY_DELETE);
    buildFrom(deleteQuery, sb2);
    sb.insert(0, sb2.toString());
  }

  public void buildFrom(IDeleteQuery deleteQuery, StringBuffer sb) throws QueryException
  {
    IFrom from = deleteQuery.getFrom();
    if (from != null)
    {
      from.buildString(this, sb);
    }
    else
    {
      String tableName = getTableName();
      sb.append(SQL_QUERY_FROM);
      sb.append(tableName);
      sb.append(SQL_QUERY_SPACE);
      sb.append(getTableAlias(tableName));
      Boolean predicateIsNull = (Boolean) objectMapping.get(PREDICATE_IS_NULL);
      if (predicateIsNull == null || !predicateIsNull.booleanValue())
      {
        sb.append(SQL_QUERY_SPACE);
        sb.append(SQL_QUERY_WHERE);
      }
    }
  }

  public void buildFrom(ISelectQuery selectQuery, StringBuffer sb) throws QueryException
  {
    IFrom from = selectQuery.getFrom();
    if (from != null)
    {
      from.buildString(this, sb);
    }
    else
    {
      String tableName = getTableName();
      sb.append(SQL_QUERY_FROM);
      sb.append(tableName);
      sb.append(SQL_QUERY_SPACE);
      sb.append(getTableAlias(tableName));
      Set multiValuePropertyNames = (Set) objectMapping.get(MULTIVALUE_PROPERTY_NAMES);
      if (multiValuePropertyNames != null)
      {
        buildMultiValueJoins(multiValuePropertyNames, sb);
      }
      else
      {
        Boolean predicateIsNull = (Boolean) objectMapping.get(PREDICATE_IS_NULL);
        if (predicateIsNull == null || !predicateIsNull.booleanValue())
        {
          sb.append(SQL_QUERY_SPACE);
          sb.append(SQL_QUERY_WHERE);
        }
      }
    }
  }

  protected void buildMultiValueJoins(Set multiValuePropertyNames, StringBuffer sb) throws QueryException
  {
    for (Iterator iter = multiValuePropertyNames.iterator(); iter.hasNext();)
    {
      String nextPropertyName = (String) iter.next();
      IJoin join = (IJoin)nameMapping.get(nextPropertyName);
      join.buildString(this, sb);
    }
    StringBuffer joinConditionsStringBuffer = (StringBuffer) objectMapping.get(JOIN_CONDITIONS);
    sb.append(joinConditionsStringBuffer.toString());
    Boolean predicateIsNull = (Boolean) objectMapping.get(PREDICATE_IS_NULL);
    if (predicateIsNull == null || !predicateIsNull.booleanValue())
    {
      sb.append(SQL_QUERY_SPACEAND);
    }
  }

  public void buildSelect(ISelectQuery selectQuery, StringBuffer sb) throws QueryException
  {
    ISelect select = selectQuery.getSelect();
    if (select != null)
    {
      select.buildString(this, sb);
    }
    else
    {
      List selectAttributes = (List) objectMapping.get(SELECT_ATTRIBUTES);
      if (selectAttributes == null)
      {
        sb.append(SQL_QUERY_SELECT_ALL);
      }
      else
      {
        sb.append(SQL_QUERY_SELECT);
        for (int i = 0; i < selectAttributes.size(); i++)
        {
          if (i > 0)
          {
            sb.append(SQL_QUERY_COMMA);
            sb.append(SQL_QUERY_SPACE);
          }
          IAttribute attribute = (IAttribute) selectAttributes.get(i);
          attribute.buildString(this, sb);
        }
        sb.append(SQL_QUERY_SPACE);
      }
    }
  }

  public void buildFrom(IFrom from, StringBuffer sb) throws QueryException
  {
    ITable table = from.getTable();
    if (table != null)
    {
      sb.append(SQL_QUERY_FROM);
      if (table instanceof ISelectQuery)
      {
        buildInnerSelect((ISelectQuery) table, sb);
        sb.append(getTableAlias(table.getName()));
      }
      else
      {
        table.buildString(this, sb);
      }
    }
    else
    {
      String tableName = getTableName();
      sb.append(SQL_QUERY_FROM);
      sb.append(tableName);
      sb.append(SQL_QUERY_SPACE);
      sb.append(getTableAlias(tableName));
    }

    IJoin[] joins = from.getJoins();
    if (joins != null && joins.length > 0)
    {
      for (int i = 0; i < joins.length; i++)
      {
        joins[i].buildString(this, sb);
      }
      Set multiValuePropertyNames = (Set) objectMapping.get(MULTIVALUE_PROPERTY_NAMES);
      if (multiValuePropertyNames != null)
      {
        for (Iterator iter = multiValuePropertyNames.iterator(); iter.hasNext();)
        {
          String nextPropertyName = (String) iter.next();
          IJoin join = (IJoin) getNameMapping(objectMapping).get(nextPropertyName);
          join.buildString(this, sb);
        }
      }
      StringBuffer joinConditionsStringBuffer = (StringBuffer) objectMapping.get(JOIN_CONDITIONS);
      sb.append(joinConditionsStringBuffer);
      Boolean predicateIsNull = (Boolean) objectMapping.get(PREDICATE_IS_NULL);
      if (predicateIsNull == null || !predicateIsNull.booleanValue())
      {
        sb.append(SQL_QUERY_SPACEAND);
      }
    }
    else
    {
      Set multiValuePropertyNames = (Set) objectMapping.get(MULTIVALUE_PROPERTY_NAMES);
      if (multiValuePropertyNames != null)
      {
        buildMultiValueJoins(multiValuePropertyNames, sb);
      }
      else
      {
        Boolean predicateIsNull = (Boolean) objectMapping.get(PREDICATE_IS_NULL);
        if (predicateIsNull == null || !predicateIsNull.booleanValue())
        {
          sb.append(SQL_QUERY_SPACE);
          sb.append(SQL_QUERY_WHERE);
        }
      }
    }
  }

  protected void buildInnerSelect(ISelectQuery table, StringBuffer sb) throws QueryException
  {
    sb.append(SQL_QUERY_LEFTPAREN);
    StringBuffer sb2 = new StringBuffer();
    Map metadata2 = new HashMap();
    metadata2.put(LAST_TABLE_ALIAS_INTEGER, objectMapping.get(LAST_TABLE_ALIAS_INTEGER));

    // use current table as primary table by default (will be overriden with from clause)    
    metadata2.put(TABLE_NAME, objectMapping.get(TABLE_NAME));
    table.buildString(this, sb2);

    // get the name of the inner select
    Map tableMap2 = (Map) metadata2.get(TABLE_MAP);
    if (tableMap2 != null)
    {
      String tableAlias = (String) tableMap2.get(table.getName());
      if (tableAlias != null)
      {
        Map tableMap = (Map) objectMapping.get(TABLE_MAP);
        if (tableMap != null)
        {
          tableMap.put(table.getName(), tableAlias);
        }
      }
    }
    objectMapping.put(LAST_TABLE_ALIAS_INTEGER, metadata2.get(LAST_TABLE_ALIAS_INTEGER));
    sb.append(sb2.toString());
    sb.append(SQL_QUERY_RIGHTPAREN);
    sb.append(SQL_QUERY_SPACE);
  }

  public void buildJoin(IJoin join, StringBuffer sb) throws QueryException
  {
    Set joins = (Set) objectMapping.get(JOINS);
    if (joins == null)
    {
      joins = new HashSet();
      objectMapping.put(JOINS, joins);
    }

    if (!joins.contains(join))
    {
      joins.add(join);
      ITable table = join.getTable();
      if (table instanceof ISelectQuery)
      {
        sb.append(SQL_QUERY_COMMA);
        sb.append(SQL_QUERY_SPACE);
        buildInnerSelect((ISelectQuery) table, sb);
        sb.append(getTableAlias(table.getName()));
      }
      else
      {
        Set joinTables = (Set) objectMapping.get(JOIN_TABLES);
        if (joinTables == null)
        {
          joinTables = new HashSet();
		  objectMapping.put(JOIN_TABLES, joinTables);
        }
        if (!joinTables.contains(table))
        {
          sb.append(SQL_QUERY_COMMA);
          sb.append(SQL_QUERY_SPACE);
          table.buildString(this, sb);
          joinTables.add(table);
        }
      }

      StringBuffer joinConditionsStringBuffer = (StringBuffer) objectMapping.get(JOIN_CONDITIONS);
      if (joinConditionsStringBuffer == null)
      {
        joinConditionsStringBuffer = new StringBuffer();
		objectMapping.put(JOIN_CONDITIONS, joinConditionsStringBuffer);
        joinConditionsStringBuffer.append(SQL_QUERY_SPACE);
        joinConditionsStringBuffer.append(SQL_QUERY_WHERE);
      }
      else
      {
        joinConditionsStringBuffer.append(SQL_QUERY_SPACEAND);
      }
      joinConditionsStringBuffer.append(SQL_QUERY_LEFTPAREN);
      join.getFromAttribute().buildString(this, joinConditionsStringBuffer);
      joinConditionsStringBuffer.append(SQL_QUERY_EQ);
      join.getToAttribute().buildString(this, joinConditionsStringBuffer);
      joinConditionsStringBuffer.append(SQL_QUERY_RIGHTPAREN);
    }
  }

  public void buildTable(ITable table, StringBuffer sb) throws QueryException
  {
    sb.append(table.getName() + SQL_QUERY_SPACE + getTableAlias(table.getName()));
  }

  public void buildSelect(ISelect select, StringBuffer sb) throws QueryException
  {
    List attributes = select.getAttributes();
    if (attributes == null)
    {
      sb.append(SQL_QUERY_SELECT_ALL);
    }
    else
    {
      sb.append(SQL_QUERY_SELECT);
      for (int i = 0; i < attributes.size(); i++)
      {
        if (i > 0)
        {
          sb.append(SQL_QUERY_COMMA);
          sb.append(SQL_QUERY_SPACE);
        }
        IAttribute attribute = (IAttribute) attributes.get(i);
        attribute.buildString(this, sb);
      }
      sb.append(SQL_QUERY_SPACE);     
    }
  }

  /**
  	<P>Returns the SQL condition string for the given condition that involves a multi-value left
  	operand and a single-value operator.
  
  	<P>This method calls processSMVSingle(), or processSMVBetween(), or processSMVIn to generate
  	the condition query string in SQL.
   */
  protected void buildSMVCondition(ICondition condition, StringBuffer sb) throws QueryException
  {
    sb.append(SQL_QUERY_CONDITIONHEAD);

    String operatorName = condition.getOperator().getName();

    if (MULTIVALUE_SINGLEOPERAND_OPERATOR_SET.contains(operatorName))
    {
      processSMVSingle(condition, sb);
    }
    else if (operatorName.equals(Constants.OPERATOR_BTW))
    {
      processSMVBetween(condition, sb);
    }
    else if (operatorName.equals(Constants.OPERATOR_IN))
    {
      processSMVIn(condition, sb);
    }
    else
    {
      throw new QueryException("Operator '" + operatorName + "' not supported.", 1);
    }
    sb.append(SQL_QUERY_CONDITIONTAIL);
  }

  /**
  	<P>Returns the SQL value string for the given value.
  
  	<P>This method generates the value string in SQL based on the data type.
   */
  public String buildValue(IValue value) throws QueryException
  {
     StringBuffer buf = new StringBuffer();
     buildValue(value, buf);
     return buf.toString();
  }
  public void buildValue(IValue value, StringBuffer sb) throws QueryException
  {
    String s = value.getValueString();
    if (s != null && s.indexOf("'") != -1)
    {
      s = makeQuerySafeValue(s);
    }

    int dataType = value.getDataType();
    if (typeMapping != null && dataType == Constants.DATATYPE_UNKNOWN)
    {
      String propertyName = value.getPropertyName();
      if (propertyName != null)
      {
        Object o = typeMapping.get(propertyName);
        if (o != null && o instanceof Integer)
        {
          dataType = ((Integer) o).intValue();
        }
      }
    }

    switch (dataType)
    {
      case Constants.DATATYPE_BIGINT :
      case Constants.DATATYPE_DECIMAL :
      case Constants.DATATYPE_DOUBLE :
      case Constants.DATATYPE_FLOAT :
      case Constants.DATATYPE_TINYINT :
      case Constants.DATATYPE_NUMERIC :
      case Constants.DATATYPE_INTEGER :
      case Constants.DATATYPE_SMALLINT :
      case Constants.DATATYPE_REAL :
      case Constants.DATATYPE_BINARY :
      case Constants.DATATYPE_UNKNOWN :
      case Constants.DATATYPE_NULL :
        break;
      case Constants.DATATYPE_CHAR :
      case Constants.DATATYPE_LONGVARCHAR :
      case Constants.DATATYPE_VARCHAR :
      case Constants.DATATYPE_DATE :
      case Constants.DATATYPE_TIME :
      case Constants.DATATYPE_TIMESTAMP :
        s = SQL_QUERY_STRINGSEP + s + SQL_QUERY_STRINGSEP;
        break;
      default :
        throw new QueryException("Data type not supported.", 3);
    }
    sb.append(s);
  }

  public Map getNameMapping(Map metadata)
  {
    Map nameMapping = (Map) metadata.get(NAME_MAPPING);
    if (nameMapping == null)
    {
      nameMapping = this.nameMapping;
    }
    return nameMapping;
  }

  public Map getTypeMapping(Map metadata)
  {
    Map typeMapping = (Map) metadata.get(TYPE_MAPPING);
    if (typeMapping == null)
    {
      typeMapping = this.typeMapping;
    }
    return typeMapping;
  }

  /**
  	Returns true if the given attribute is a multi-value attribute. False otherwise.
   */
  public boolean isMultiValueAttribute(IAttribute a)
  {
    boolean isMultiValueAttribute = false;
    if ((nameMapping != null) && (nameMapping.containsKey(a.getName())))
    {
      Object o = nameMapping.get(a.getName());
      if (o instanceof IJoin || o instanceof String[])
      {
        isMultiValueAttribute = true;
      }
    }
    return isMultiValueAttribute;
  }

  /**
     <P>Returns the SQL condition string for operator "MVALL" in conjunction with the given left
     operand and right operands.

     <P>This method converts a MVALL condition into a sub query condition string in SQL. The left
     operand has to be a multi-value attribute.
   */
  protected void processMvAll(ILeftOperand l, IRightOperand[] r, StringBuffer sb) throws QueryException {
     if ((l instanceof IAttribute) == false)
        throw new QueryException("Left operand for multi value operation must be an attribute.", 4);

     boolean isRightOperandsNull = false;
     if (r.length == 0 || r[0] == null) {
        isRightOperandsNull = true;
     }
     else if (r[0] instanceof Value) {
        isRightOperandsNull = ((Value) r[0]).getValueString() == null;
     }

     IAttribute a = (IAttribute) l;
     String mName = a.getName();

     if ((nameMapping != null) && (nameMapping.containsKey(mName))) {
        if (!isMultiValueAttribute(a)) {
           throw new QueryException("Left operand for multi value operation must be a multi-value attribute.", 4);
        }

		String aTable = null;
		String aName = null;
		String aId = null;
		String mId = null;

		Object nameMappingObject = nameMapping.get(mName);
		if (nameMappingObject instanceof String[]) {
			String[] sa = (String[]) nameMapping.get(mName);
			aTable = sa[0];
			aName = sa[1];
			aId = sa[2];
			mId = sa[3];
		} else {
			IJoin join = (IJoin) nameMapping.get(mName);
			aName = join.getCompareAttribute().getName();
			aId = join.getToAttribute().getName();
			mId = join.getFromAttribute().getName();
		}


        if (isRightOperandsNull) {
           sb.append(SQL_QUERY_LEFTPAREN); sb.append(mId);
           sb.append(SQL_QUERY_SPACE); sb.append(SQL_QUERY_IS); sb.append(SQL_QUERY_SPACE); sb.append(SQL_QUERY_NULL);
           sb.append(SQL_QUERY_RIGHTPAREN);
           sb.append(SQL_QUERY_SPACE); sb.append(SQL_QUERY_AND); sb.append(SQL_QUERY_SPACE);
           sb.append(SQL_QUERY_LEFTPAREN); sb.append(mId);
           sb.append(SQL_QUERY_SPACE); sb.append(SQL_QUERY_ISN); sb.append(SQL_QUERY_SPACE); sb.append(SQL_QUERY_NULL);
           sb.append(SQL_QUERY_RIGHTPAREN);
        }
        else {
           for (int i = 0; i < r.length; i++) {
              if (i > 0) {
                 sb.append(SQL_QUERY_SPACE); sb.append(SQL_QUERY_AND); sb.append(SQL_QUERY_SPACE);
              }
              // process multivalue condition as subselect
              sb.append(SQL_QUERY_LEFTPAREN); sb.append(mId); sb.append(SQL_QUERY_SPACE); sb.append(SQL_QUERY_IN);
              sb.append(SQL_QUERY_SPACE); sb.append(SQL_QUERY_LEFTPAREN);
              sb.append(SQL_QUERY_SELECT); sb.append(SQL_QUERY_SPACE); sb.append(aId); sb.append(SQL_QUERY_SPACE);
              sb.append(SQL_QUERY_FROM); sb.append(SQL_QUERY_SPACE); sb.append(aTable); sb.append(SQL_QUERY_SPACE);
              sb.append(SQL_QUERY_WHERE); sb.append(SQL_QUERY_SPACE); sb.append(aName);
              sb.append(SQL_QUERY_EQ);
              r[i].buildString(this, sb);
              sb.append(SQL_QUERY_RIGHTPAREN); sb.append(SQL_QUERY_RIGHTPAREN);

              // process multivalue condition as query on joined table
              /*				s = s + SQL_QUERY_LEFTPAREN + aName;
                          s = s + SQL_QUERY_EQ + r[i].buildString(this) + SQL_QUERY_RIGHTPAREN;*/

           }
        }
     }
     else {
        // No nameMappings, use Personalization 3.5 syntax
        l.buildString(this, sb);
        sb.append(SQL_QUERY_SPACE); sb.append(SQL_QUERY_MVALL); sb.append(SQL_QUERY_SPACE); sb.append(SQL_QUERY_LEFTPAREN);
        for (int i = 0; i < r.length; i++) {
           if (i > 0)
              sb.append(SQL_QUERY_COMMA);
           r[i].buildString(this, sb);
        }
        sb.append(SQL_QUERY_RIGHTPAREN);
     }

  }
  /**
     <P>Returns the SQL condition string for operator "MVANY" in conjunction with the given left
     operand and right operands.

     <P>This method converts a MVANY condition into a sub query condition string in SQL. The left
     operand has to be a multi-value attribute.
   */
  protected void processMvAny(ILeftOperand l, IRightOperand[] r, StringBuffer sb) throws QueryException {
     String s = "";
     if ((l instanceof IAttribute) == false)
        throw new QueryException("Left operand for multi value operation must be an attribute.", 4);

     boolean isRightOperandsNull = false;
     if (r.length == 0 || r[0] == null) {
        isRightOperandsNull = true;
     }
     else if (r[0] instanceof Value) {
        isRightOperandsNull = ((Value) r[0]).getValueString() == null;
     }

     IAttribute a = (IAttribute) l;
     String mName = a.getName();

     if ((nameMapping != null) && (nameMapping.containsKey(mName))) {
        if (!isMultiValueAttribute(a)) {
           throw new QueryException("Left operand for multi value operation must be a multi-value attribute.", 4);
        }

        String[] sa = (String[]) nameMapping.get(mName);
        //String aTable = sa[0];
        String aName = sa[1];
        //String aId = sa[2];
        //String mId = sa[3];

        if (isRightOperandsNull) {
           // process multivalue condition as subselect
           /*			s = mId + SQL_QUERY_SPACE + SQL_QUERY_IN + SQL_QUERY_SPACE + SQL_QUERY_LEFTPAREN;
                    s = s + SQL_QUERY_SELECT + SQL_QUERY_SPACE + aId + SQL_QUERY_SPACE;
                    s = s + SQL_QUERY_FROM + SQL_QUERY_SPACE + aTable + SQL_QUERY_SPACE;
                    s = s + SQL_QUERY_WHERE + SQL_QUERY_SPACE + SQL_QUERY_LEFTPAREN + SQL_QUERY_LEFTPAREN + aName;
                    s = s + SQL_QUERY_SPACE + SQL_QUERY_IS + SQL_QUERY_SPACE + SQL_QUERY_NULL;
                    s = s + SQL_QUERY_RIGHTPAREN;
                    s = s + SQL_QUERY_SPACE + SQL_QUERY_AND + SQL_QUERY_SPACE;
                    s = s + SQL_QUERY_LEFTPAREN + aName;
                    s = s + SQL_QUERY_SPACE + SQL_QUERY_ISN + SQL_QUERY_SPACE + SQL_QUERY_NULL;
                    s = s + SQL_QUERY_RIGHTPAREN + SQL_QUERY_RIGHTPAREN + SQL_QUERY_RIGHTPAREN;
           */
           // process multivalue condition as query on joined table
           sb.append(SQL_QUERY_LEFTPAREN); sb.append(SQL_QUERY_LEFTPAREN); sb.append(aName);
           sb.append(SQL_QUERY_SPACE); sb.append(SQL_QUERY_IS); sb.append(SQL_QUERY_SPACE); sb.append(SQL_QUERY_NULL);
           sb.append(SQL_QUERY_RIGHTPAREN);
           sb.append(SQL_QUERY_SPACE); sb.append(SQL_QUERY_AND); sb.append(SQL_QUERY_SPACE);
           sb.append(SQL_QUERY_LEFTPAREN); sb.append(aName);
           sb.append(SQL_QUERY_SPACE); sb.append(SQL_QUERY_ISN); sb.append(SQL_QUERY_SPACE); sb.append(SQL_QUERY_NULL);
           sb.append(SQL_QUERY_RIGHTPAREN); sb.append(SQL_QUERY_RIGHTPAREN);
        }
        else {
           // process multivalue condition as subselect
           /*			s = mId + SQL_QUERY_SPACE + SQL_QUERY_IN + SQL_QUERY_SPACE + SQL_QUERY_LEFTPAREN;
                    s = s + SQL_QUERY_SELECT + SQL_QUERY_SPACE + aId + SQL_QUERY_SPACE;
                    s = s + SQL_QUERY_FROM + SQL_QUERY_SPACE + aTable + SQL_QUERY_SPACE;
                    s = s + SQL_QUERY_WHERE + SQL_QUERY_SPACE + aName + SQL_QUERY_SPACE;
                    s = s + SQL_QUERY_IN + SQL_QUERY_SPACE + SQL_QUERY_LEFTPAREN;
                    for (int i=0; i < r.length; i++) {
                       if (i > 0)
                          s = s + SQL_QUERY_COMMA;

                       s = s + r[i].buildString(this);
                    }
                    s = s + SQL_QUERY_RIGHTPAREN + SQL_QUERY_RIGHTPAREN;
           */
           // process multivalue condition as query on joined table
           sb.append(aName); sb.append(SQL_QUERY_SPACE);
           sb.append(SQL_QUERY_IN); sb.append(SQL_QUERY_SPACE); sb.append(SQL_QUERY_LEFTPAREN);
           for (int i = 0; i < r.length; i++) {
              if (i > 0)
                 sb.append(SQL_QUERY_COMMA);

              r[i].buildString(this, sb);
           }
           sb.append(SQL_QUERY_RIGHTPAREN);
        }
     }
     else {
        // No nameMappings, use Personalization 3.5 syntax
        l.buildString(this,sb); sb.append(SQL_QUERY_SPACE); sb.append(SQL_QUERY_MVANY); sb.append(SQL_QUERY_SPACE); sb.append(SQL_QUERY_LEFTPAREN);
        for (int i = 0; i < r.length; i++) {
           if (i > 0)
              sb.append(SQL_QUERY_COMMA);
           r[i].buildString(this, sb);
        }
        sb.append(SQL_QUERY_RIGHTPAREN);
     }

  }
  /**
  	<P>Returns the SQL condition string for the given condition that involves a multi-value left
  	operand, a "BTW" operator, and two right operands.
  
  	<P>This method converts a multi-value BTW condition into a sub query condition string in SQL.
  	The left operand has to be a multi-value attribute.
   */
  protected void processSMVBetween(ICondition condition, StringBuffer sb) throws QueryException
  {
    IOperator o = condition.getOperator();
    ILeftOperand l = condition.getLeftOperand();
    IRightOperand[] r = condition.getRightOperands();

    IAttribute a = (IAttribute) l;
    String mName = a.getName();

    if ((nameMapping != null) && (nameMapping.containsKey(mName)))
    {
      IJoin join = (IJoin) nameMapping.get(mName);
      // process multivalue condition as query on joined table            
      join.getCompareAttribute().buildString(this, sb);
      sb.append(SQL_QUERY_SPACE);
      o.buildString(this, sb);
      sb.append(SQL_QUERY_SPACE);
      r[0].buildString(this, sb);
      sb.append(SQL_QUERY_SPACEAND);
      r[1].buildString(this, sb);
    }
    else
    {
      throw new QueryException();
    }
  }
  /**
  	<P>Returns the SQL condition string for the given condition that involves a multi-value left
  	operand, an "IN" operator, and multiple right operands.
  
  	<P>This method converts a multi-value IN condition into a sub query condition string in SQL.
  	The left operand has to be a multi-value attribute.
   */
  protected void processSMVIn(ICondition condition, StringBuffer sb) throws QueryException
  {
    ILeftOperand l = condition.getLeftOperand();
    IRightOperand[] r = condition.getRightOperands();

    boolean isRightOperandsNull = false;
    if (r.length == 0 || r[0] == null)
    {
      isRightOperandsNull = true;
    }
    else if (r[0] instanceof Value)
    {
      isRightOperandsNull = ((Value) r[0]).getValueString() == null;
    }

    IAttribute a = (IAttribute) l;
    String mName = a.getName();

    if ((nameMapping != null) && (nameMapping.containsKey(mName)))
    {
      IJoin join = (IJoin) nameMapping.get(mName);

      StringBuffer sb2 = new StringBuffer();
      join.getCompareAttribute().buildString(this, sb2);
      String aName = sb2.toString();

      if (isRightOperandsNull)
      {
        // process multivalue condition as query on joined table
        sb.append(SQL_QUERY_LEFTPAREN + aName);
        sb.append(SQL_QUERY_ISNULL);
        sb.append(SQL_QUERY_RIGHTPAREN);
        sb.append(SQL_QUERY_SPACEAND);
        sb.append(SQL_QUERY_LEFTPAREN + aName);
        sb.append(SQL_QUERY_ISNNULL);
        sb.append(SQL_QUERY_RIGHTPAREN);
      }
      else
      {
        // process multivalue condition as query on joined table
        sb.append(aName + SQL_QUERY_SPACE);
        sb.append(SQL_QUERY_IN + SQL_QUERY_SPACE + SQL_QUERY_LEFTPAREN);
        for (int i = 0; i < r.length; i++)
        {
          if (i > 0)
          {
            sb.append(SQL_QUERY_COMMA);
          }
          r[i].buildString(this, sb);
        }
        sb.append(SQL_QUERY_RIGHTPAREN);
      }
    }
    else
    {
      throw new QueryException();
    }
  }
  /**
  	<P>Returns the SQL condition string for the given condition that involves a multi-value left
  	operand, a single-value operator, and a single right operand.
  
  	<P>This method converts a multi-value condition that takes a single right operand into a sub
  	query condition string in SQL. The left operand has to be a multi-value attribute.
   */
  protected void processSMVSingle(ICondition condition, StringBuffer sb) throws QueryException
  {
    IOperator o = condition.getOperator();
    ILeftOperand l = condition.getLeftOperand();
    IRightOperand[] r = condition.getRightOperands();

    IAttribute a = (IAttribute) l;
    String mName = a.getName();

    String operatorName = o.getName();

    if (operatorName.equals(Constants.OPERATOR_LIKE) || operatorName.equals(Constants.OPERATOR_NOT_LIKE))
    {
      if (r[0] instanceof Value)
      {
        Value v = (Value) r[0];
        int dt = v.getDataType();

        if ((dt == Constants.DATATYPE_CHAR) || (dt == Constants.DATATYPE_LONGVARCHAR) || (dt == Constants.DATATYPE_VARCHAR))
        {

          if ((nameMapping != null) && (nameMapping.containsKey(mName)))
          {
            IJoin join = (IJoin) nameMapping.get(mName);
            // process multivalue condition as query on joined table
            join.getCompareAttribute().buildString(this, sb);
            sb.append(SQL_QUERY_SPACE);
            o.buildString(this, sb);
            sb.append(SQL_QUERY_SPACE);
            int len = sb.length();
            v.buildString(this, sb);
            String vs = v.getValueString();
            if (vs.indexOf(SQL_QUERY_WILDCARD) == -1) {
               sb.insert(len + 1, SQL_QUERY_WILDCARD);
               sb.insert(sb.length() - 1, SQL_QUERY_WILDCARD);
            }

          }
          else
          {
            throw new QueryException();
          }
        }
        else
        {
          throw new QueryException("Invalid data type for operator '" + operatorName + "'.", 1);
        }
      }
      else
      {
        throw new QueryException("Operator '" + operatorName + "' not supported in this context.", 1);
      }
    }
    else
    {
      if ((nameMapping != null) && (nameMapping.containsKey(mName)))
      {
        IJoin join = (IJoin) nameMapping.get(mName);

        // process multivalue condition as query on joined table
        sb.append(SQL_QUERY_LEFTPAREN);
        join.getCompareAttribute().buildString(this, sb);
        sb.append(SQL_QUERY_SPACE);
        o.buildString(this, sb);
        sb.append(SQL_QUERY_SPACE);
        r[0].buildString(this, sb);
        sb.append(SQL_QUERY_RIGHTPAREN);
      }
      else
      {
        throw new QueryException();
      }
    }
  }

  public String makeQuerySafeValue(String myValueString)
  {
    java.util.StringTokenizer st = new java.util.StringTokenizer(myValueString, "'", true);
    StringBuffer sb = new StringBuffer(myValueString.length() * 2);

    while (st.hasMoreTokens())
    {
      String s = st.nextToken();

      if (s.equals(SQL_QUERY_STRINGSEP))
      {
        sb.append(s + s);
      }
      else
      {
        sb.append(s);
      }
    }
    return sb.toString();
  }
  /**
   * @see ISelectQueryCallback#buildArithmeticCondition(IArithmeticCondition)
   */

  public String buildArithmeticCondition(IArithmeticCondition arithmeticCondition) throws QueryException
  {
     StringBuffer buf = new StringBuffer();
     buildArithmeticCondition(arithmeticCondition, buf);
     return buf.toString();
  }

  public void buildArithmeticCondition(IArithmeticCondition arithmeticCondition, StringBuffer sb) throws QueryException
  {
    sb.append(SQL_QUERY_SPACE);
    String arithmeticOperation = arithmeticCondition.getArithmeticOperation();
    ILeftOperand[] arithmeticOperands = arithmeticCondition.getArithmeticConditionElements();
    if (arithmeticOperation.equals(SQL_QUERY_ARITHMETIC_DIVIDE))
    {
      sb.append("case when ");
      for (int i = 1; i < arithmeticOperands.length; i++)
      {
        if (i > 1)
        {
          sb.append(SQL_QUERY_SPACEOR);
        }
        if (arithmeticOperands[i] instanceof Attribute)
        {
          ((Attribute) arithmeticOperands[i]).buildString(this, sb);
          sb.append(" = 0");
          sb.append(SQL_QUERY_SPACEOR);
          ((Attribute) arithmeticOperands[i]).buildString(this, sb);
          sb.append(" is null");
        }
        else if (arithmeticOperands[i] instanceof Function)
        {
          ((Function) arithmeticOperands[i]).buildString(this, sb);
          sb.append(" = 0");
          sb.append(SQL_QUERY_SPACEOR);
          ((Function) arithmeticOperands[i]).buildString(this, sb);
          sb.append(" is null");
        }
        else if (arithmeticOperands[i] instanceof Value)
        {
          ((Value) arithmeticOperands[i]).buildString(this, sb);
          sb.append(" = 0");
          sb.append(SQL_QUERY_SPACEOR);
          ((Value) arithmeticOperands[i]).buildString(this, sb);
          sb.append(" is null");
        }
        else if (arithmeticOperands[i] instanceof ArithmeticCondition)
        {
          sb.append(SQL_QUERY_LEFTPAREN);
          ((ArithmeticCondition) arithmeticOperands[i]).buildString(this, sb);
          sb.append(SQL_QUERY_RIGHTPAREN + " = 0");
          sb.append(SQL_QUERY_SPACEOR + SQL_QUERY_LEFTPAREN);
          ((ArithmeticCondition) arithmeticOperands[i]).buildString(this, sb);
          sb.append(SQL_QUERY_RIGHTPAREN + " is null");
        }
      }
      sb.append(" then null else ");

    }
    for (int i = 0; i < arithmeticOperands.length; i++)
    {
      if (i > 0)
      {
        sb.append(SQL_QUERY_SPACE + arithmeticOperation + SQL_QUERY_SPACE);
      }
      if (arithmeticOperands[i] instanceof Attribute)
      {
        ((Attribute) arithmeticOperands[i]).buildString(this, sb);
      }
      else if (arithmeticOperands[i] instanceof Function)
      {
        ((Function) arithmeticOperands[i]).buildString(this, sb);
      }
      else if (arithmeticOperands[i] instanceof Value)
      {
        ((Value) arithmeticOperands[i]).buildString(this, sb);
      }
      else if (arithmeticOperands[i] instanceof ArithmeticCondition)
      {
        sb.append(SQL_QUERY_LEFTPAREN);
        ((ArithmeticCondition) arithmeticOperands[i]).buildString(this, sb);
        sb.append(SQL_QUERY_RIGHTPAREN);
      }
    }
    if (arithmeticOperation.equals(SQL_QUERY_ARITHMETIC_DIVIDE))
    {
      sb.append(" end");
    }
    sb.append(SQL_QUERY_SPACE);
  }

  /**
   * @see ISelectQueryCallback#buildLimit(ILimit)
   */
  public String buildLimit(ILimit limit) throws QueryException
  {
     StringBuffer buf = new StringBuffer();
     buildLimit(limit, buf);
     return buf.toString();
  }
  public void buildLimit(ILimit limit, StringBuffer sb) throws QueryException
  {
    sb.append(limit.getLimitValue());
  }

  /**
   * @see ISelectQueryCallback#buildLimit(ISelectQuery)
   */
  public String buildLimit(ISelectQuery selectQuery) throws QueryException
  {
     StringBuffer buf = new StringBuffer();
     buildLimit(selectQuery, buf);
     return buf.toString();
  }
  public void buildLimit(ISelectQuery selectQuery, StringBuffer sb) throws QueryException
  {
  }

  protected String getTableAlias(String tableName)
  {
    Map tableMap = (Map) objectMapping.get(TABLE_MAP);
    String tableAlias = null;
    if (tableMap == null)
    {
      tableMap = new HashMap();
      objectMapping.put(TABLE_MAP, tableMap);
    }
    else if (tableName != null)
    {
      tableAlias = (String) tableMap.get(tableName);
    }
    if (tableAlias == null)
    {
      Integer lastTableAliasInteger = (Integer) objectMapping.get(LAST_TABLE_ALIAS_INTEGER);
      if (lastTableAliasInteger == null)
      {
        lastTableAliasInteger = new Integer(0);
        objectMapping.put(LAST_TABLE_ALIAS_INTEGER, lastTableAliasInteger);
      }
      else
      {
        lastTableAliasInteger = new Integer(lastTableAliasInteger.intValue() + 1);
        objectMapping.put(LAST_TABLE_ALIAS_INTEGER, lastTableAliasInteger);
      }
      if (tableName != null)
      {
        tableAlias = "T" + lastTableAliasInteger;
        tableMap.put(tableName, tableAlias);
      }
      else
      {
        tableAlias = "T" + lastTableAliasInteger;
        tableMap.put(tableAlias, tableAlias);
      }
    }
    return tableAlias;
  }

  protected String getTableName() throws QueryException
  {
     return (String) objectMapping.get(TABLE_NAME);
  }

}
