/*
 * Ticket.java
 *
 * This file is part of the specification for the
 * Java Content Repository API
 */
package javax.jcr;

import javax.jcr.observation.ObservationManager;

/**
 * Through this interface all access to the content repository occurs.
 * Given upon <code>connect</code> to a <code>Workspace</code>.
 * Both level 1 and 2 implementations must implement this interface, though
 * in level 1 some methods will throw an
 * <code>UnsupportedRepositoryOperationException</code> (see the
 * method-by-method descriptions).
 * @see javax.jcr.xa.XATicket
 * @author Peeter Piegaze
 * @author Stefan Guggisberg
 */
public interface Ticket {
    /**
     * Gets the repository that issued this ticket.
     * Returns the <code>Repository</code> object through which this ticket was
     * acquired.
     *
     * @return a <code>{@link Repository}</code> object.
     */
    public Repository getRepository();

    /**
     * Returns the Credentials object that was used to authorize the issue of
     * this ticket.
     * Each ticket is associated with a single user, though a user may have
     * more than one ticket.
     *
     * @return the Credentials of this ticket.
     */
    public Credentials getCredentials();

    /**
     * Returns new ticket in accordance with the specified
     * (new) Credentials.
     * In effect allows the current user to "impersonate"
     * another use (assuming that their original ticket gives them that right).
     *
     * @param credentials A <code>Credentials</code> object
     *
     * @throws LoginException
     */
    public Ticket impersonate(Credentials credentials) throws LoginException;
    
    /**
     * Returns a Workspace object representing the content repository
     * as "seen" by the this Ticket.
     *
     * @return a Workspace object
     * @throws RepositoryException if an error occurs
     */
    public Workspace getWorkspace() throws RepositoryException;

    /**
     * Returns the specified workspace if this ticket has permission to
     * access it, otherwise throw <code>AccessDeniedException</code>.
     *
     * @param workspaceName
     * @return a Workspace object
     * @throws RepositoryException if there is no workspace with specified name
     */
    public Workspace getWorkspace(String workspaceName) throws RepositoryException;
    
    /**
     * Returns the names of all workspaces accessible through the ticket.
     * @return  String[] An array of workspace names.
     * @throws RepositoryException if an error occurs.
     */
    public String[] getWorkspaces() throws RepositoryException;
    
    /**
     * Releases all resources associate with this ticket.  Calling this method is not mandatory 
     * since a good implementaion should automatically timeout anyway.  However, when more precise 
     * control over resource allocation is need this method can be used.
     * @throws RepositoryException
     */    
    public void logout() throws RepositoryException;
    
    /**
     * If observation is supproted gets the <code>ObservationManager</code>,
     * otherwise throws an <code>UnsupportedRepositoryOperationException</code>.
     * <p>
     * <b>Level 1:</b>
     * <p>
     * Always throws an <code>UnsupportedRepositoryOperationException</code> since
     * observation is not in supported in level 1.
     * <p>
     * <b>Level 2:</b>
     * <p>
     * If observation is supported (it is optional, even in level 2), gets the
     * <code>ObservationManager</code> object through which event observation
     * is managed. Otherwise throws an
     * <code>UnsupportedRepositoryOperationException</code>.
     *
     * @throws UnsupportedRepositoryOperationException In level 1: Always.
     * In level 2: if implementation doesnot support observation.
     *
     * @return an <code>ObservationManager</code> object.
     */
    public ObservationManager getObservationManager() throws UnsupportedRepositoryOperationException;
}