/*
 * AccessManager.java
 *
 * This file is part of the specification for the
 * Java Content Repository API
 */
package javax.jcr.access;

import javax.jcr.RepositoryException;
import javax.jcr.PathNotFoundException;

/**
 * The access manager object.
 * <p>
 * <b>Level 2 only</b>
 * <p>
 * Acquired via <code>{@link javax.jcr.Workspace#getAccessManager()}</code>.
 *
 * @author Peeter Piegaze
 * @author Stefan Guggisberg
 */
public interface AccessManager {

    /**
     * Returns the status of the item at <code>absPath</code> regarding
     * the specified <code>permission</code>s (under the rights held by current
     * ticket). See <code>{@link Permission}</code> for more details.
     *
     * @param absPath The absolute path of an item.
     *
     * @param permissions A combination of one or more of the following constants
     * encoded as a bitmask value:
     * <ul>
     * <li><code>Permission.ADD_NODE</code></li>
     * <li><code>Permission.ADD_PROPERTY</code></li>
     * <li><code>Permission.REMOVE_ITEM</code></li>
     * <li><code>Permission.READ_ITEM</code></li>
     * <li><code>Permission.WRITE_PROPERTY</code></li>
     * </ul>
     * See <code>{Permission}</code>.
     *
     * @return <code>true</code> if <code>permissions</code> are true for the
     * item at <code>absPath</code>; <code>false</code> otherwise.
     *
     * @throws PathNotFoundException if no <code>Item</code> exists at
     *         <code>absPath</code>.
     *
     * @throws RepositoryException If an unexpected error occurs.
     *
     * @throws IllegalArgumentException if <code>permissions</code> specifies
     * invalid permissions.
     */
    public boolean isGranted(String absPath, long permissions) throws PathNotFoundException, RepositoryException;

    /**
     * Returns the set of permissions that are
     * <code>true</code> for the item at <code>absPath</code>.
     *
     * @param absPath an absolute path to an item.
     *
     * @return A set of permissions encoded in a bitmask of permission constants
     * defined in <code>{@link Permission}</code>
     *
     * @throws PathNotFoundException if no <code>Item</code> exists at
     *         <code>absPath</code>.
     *
     * @throws RepositoryException If an unexpected error occurs.
     */
    public long getPermissions(String absPath) throws PathNotFoundException, RepositoryException;

    /**
     * Returns an array of <code>Permission</code> objects describing all
     * permissions supported by this repository.
     *
     * @return An array of <code>Permission</code>s.
     */
    public Permission[] getSupportedPermissions();
}
