/*
 * Permission.java
 *
 * This file is part of the specification for the
 * Java Content Repository API
 */
package javax.jcr.access;

/**
 * The permissions used by <code>{@link javax.jcr.access.AccessManager}</code>.
 * <p>
 * <b>Level 2 only</b>
 * <p>
 * <p>This interface defines the following permissions:
 * <UL>
 *    <LI>ADD_NODE
 *    <LI>ADD_PROPERTY
 *    <LI>REMOVE_ITEM
 *    <LI>READ_ITEM
 *    <LI>WRITE_PROPERTY
 * </UL>
 * <b>Level 2 only</b></code>
 * <p>
 * Given a particular ticket, a particular <code>Permission</code> is either
 * <code>true</code> or <code>false</code> of a particular node or property.
 * The possible permissions and their meanings are:
 * <dl>
 * <dt><code>Permission.ADD_NODE</code></dt>
 * <dd>
 * If <code>true</code> of node <code>N</code> then <code>N.addNode</code>
 * is allowed. If <code>false</code> then it will
 * throw an <code>AccessDeniedException</code>.
 * Node type restrictions are enforced separately from access control
 * restriction: Even if <code>Permission.ADD_NODE</code> is <code>true</code>
 * of node <code>N</code>, <code>N.addNode</code> may still throw a
 * <code>ParentChildMismatch</code> if adding specified node would violate
 * the node type of <code>N</code>. Always <code>false</code> of properties.
 * </dd>
 * <dt><code>Permission.ADD_PROPERTY</code></dt>
 * <dd>
 * If <code>true</code> of node <code>N</code> then <code>N.addProperty</code>
 * is allowed. If <code>false</code> then it will
 * throw an <code>AccessDeniedException</code>. Node type restrictions are
 * enforced separately from access control restriction: Even if
 * <code>Permission.ADD_PROPERTY</code> is <code>true</code>
 * of node <code>N</code>, <code>N.addProperty</code> may still throw a
 * <code>ParentChildMismatch</code> if adding the specified property would
 * violate the node type of <code>N</code>. Always <code>false</code> of
 * properties.
 * </dd>
 * <dt><code>Permission.REMOVE_ITEM</code></dt>
 * <dd>
 * If <code>true</code> of item (node or property) <code>E</code> then
 * <code>remove(E.getPath())</code> is allowed. If <code>false</code>, then it
 * will throw an <code>AccessDeniedException</code>.
 * </dd>
 * <dt><code>Permission.READ_ITEM</code></dt>
 * <dd>
 * If <code>true</code> of property <code>P</code> then <code>getProperty(P.getPath())</code>
 * is allowed. If <code>false</code> then it throws an
 * <code>AccessDeniedException</code>. If <code>true</code> of node
 * <code>N</code> then there is at least one property of <code>N</code> for
 * which <code>READ_ITEM</code> is <code>true</code>. If false then
 * </code>READ_ITEM</code> is </code>false for all properties of
 * <code>N</code>.
 * </dd>
 * <dt><code>Permission.WRITE_PROPERTY</code></dt>
 * <dd>
 * If <code>true</code> of property <code>P</code> then
 * <code>setProperty(P.getPath(), value)</code> is allowed. If <code>false</code>
 * then it throws <code>AccessDeniedException</code>.
 * </dd>
 * </dl>
 *
 * @author Peeter Piegaze
 * @author Stefan Guggisberg
 */
public interface Permission {

    /**
     * The permissions defined by the JCR standard. Each constant is a
     * power of 2 (i.e. sets of permissions can be encoded as a bitmask
     * in a <code>long</code> value).
     */
    public static final long ADD_NODE = 1;
    public static final long ADD_PROPERTY = 2;
    public static final long REMOVE_ITEM = 4;
    public static final long READ_ITEM = 8;
    public static final long WRITE_PROPERTY = 16;

    /**
     * Returns the numerical constant identifying this permission. This value
     * is always a power of 2 ((i.e. sets of permissions can be encoded as
     * a bitmask in a <code>long</code> value).
     * @return the numerical value
     * @see AccessManager#getSupportedPermissions
     */
    public long getValue();

    /**
     * Returns the descriptive name of this permission.
     * @return the name
     * @see AccessManager#getSupportedPermissions
     */
    public String getName();
}

