/*
* ChildNodeDef.java
*
* This file is part of the specification for the
* Java Content Repository API
*/
package javax.jcr.nodetype;

/**
 * A child node definition.
 *
 * @see NodeType#getChildNodeDefs
 * @see javax.jcr.Node#getDefinition
 *
 * @author Peeter Piegaze
 * @author Stefan Guggisberg
 */
public interface NodeDef {

    /**
     * Gets the node type that contains the declaration of <i>this</i>
     * <code>NodeDef</code>.
     *
     * @return a <code>NodeType</code> object.
     */
    public NodeType getDeclaringNodeType();

    /**
     * Gets the name of the child node. If <code>null</code>, this
     * <code>NodeDef</code> defines a residual set of child nodes. That is,
     * it defines the characteristics of all those child nodes with names apart
     * from the names explicitly used in other property or child node
     * definitions.
     *
     * @return a <code>String</code> or <code>null</code>.
     */
    public String getName();

    /**
     * Gets the node type that the child node must have.
     * The simplest case would be to return
     * <code>nt:base</code> which is the base of all node types and
     * therefore, in this context, represents the least restrictive
     * node type requirement.
     *
     * @return a <code>NodeType</code> object.
     */
    public NodeType[] getRequiredNodeTypes();

    /**
     * Gets the default node type that will be assigned to the child node if it
     * is created without an explicitly specified node type. This node type must
     * be a subtype of (or the same type as) the node type returned by
     * <code>getRequiredNodeType</code>. The simplest case would be
     * to return <code>nt:default</code> which is the defult node type
     * that places no restrictions on properties or child nodes.
     *
     * @return a <code>NodeType</code>.
     */
    public NodeType getDefaultNodeType();

    /**
     * Reports whether the child node is to be automatically created when its
     * parent node is created. If set to <code>true</code> then this
     * <code>NodeDef</code> must not be a residual set definition
     * but must specify an actual child node name. This child node
     * may be of a node type that specifies further auto-created child nodes.
     * In such a case a chain of nodes will be created.
     *
     * @return a <code>boolean</code>.
     */
    public boolean isAutoCreate();

    /**
     * Gets the on-version status of the child node. This governs what to do if
     * the parent node of this child node is versioned; an
     * <code>OnParentVersionAction</code>, one of:
     * <ul>
     *       <li><code>OnParentVersionAction#COPY</code>, or</li>
     *       <li><code>OnParentVersionAction#VERSION</code>, or</li>
     *       <li><code>OnParentVersionAction#INITIALIZE</code>, or</li>
     *       <li><code>OnParentVersionAction#COMPUTE</code>, or</li>
     *       <li><code>OnParentVersionAction#IGNORE</code>, or</li>
     *       <li><code>OnParentVersionAction#ABORT</code></li>
     * </ul>
     *
     * @see OnVersionNodeAction
     *
     * @return an <code>int</code>.
     */
    public int getOnVersion();

    /**
     * Reports whether the child node is read-only. This
     * means that this child node cannot be created or have child nodes added
     * to it via this API. It may be created or have child nodes added to it,
     * however, by the implementation itself, via some internal mechanism not
     * specified by this standard.
     *
     * @return a <code>boolean</code>.
     */
    public boolean isReadOnly();

	/**
	 * Reports whether the child node allows same-name siblings.
	 *
	 * @return a <code>boolean</code>
	 */

	public boolean isMultiple();

    /**
     * Reports whether this node is the primary child item of its parent
     * node. Since any given node may have either zero or one primary child
     * items, this means that a maximum of one <code>PropertyDef</code> or
     * <code>NodeDef</code> within a particular <code>NodeType</code> may
     * return <code>true</code> on this call. The primary child item flag
     * is used by the method <code>{@link javax.jcr.Node#getPrimaryItem()
     * Node.getPrimaryItem()}</code>.
     *
     * @return a <code>boolean</code>.
     */
    public boolean isPrimaryItem();
}