/*
 * NodeType.java
 *
 * This file is part of the specification for the
 * Java Content Repository API
 */
package javax.jcr.nodetype;

import javax.jcr.Value;
import javax.jcr.RepositoryException;
import javax.jcr.ValueFormatException;

/**
 * Represents a node type.
 *
 * @author Peeter Piegaze
 * @author Stefan Guggisberg
 */
public interface NodeType {

    /**
     * Returns the name of the node type.
     *
     * @return the name of the node type
     */
    public String getName();

    /**
     * Returns all supertypes of this node type including both those directly
     * declared and those inherited.
     *
     * @see #getDeclaredSupertypes
     *
     * @return an array of <code>NodeType</code> objects.
     */
    public NodeType[] getSupertypes();

    /**
     * Returns an array containing the property definitions of this node type,
     * including the property definitions inherited from supertypes of this node
     * type.
     *
     * @see #getDeclaredPropertyDefs
     *
     *
     * @return an array containing the property definitions.
     */
    public PropertyDef[] getPropertyDefs();

    /**
     * Returns an array containing the child node definitions of this node type,
     * including the child node definitions inherited from supertypes of this
     * node type.
     *
     * @see #getDeclaredChildNodeDefs
     *
     * @return an array containing the child node definitions.
     */
    public NodeDef[] getChildNodeDefs();

    /**
     * Returns all <i>direct</i> supertypes as specified in the declaration of
     * <i>this</i> node type. In single inheritance systems this will always be
     * an array of size 0 or 1. In systems that support multiple inheritance of
     * node types this array may be of size greater than 1.
     *
     * @see #getSupertypes
     *
     * @return an array of <code>NodeType</code> objects.
     */
    public NodeType[] getDeclaredSupertypes();

    /**
     * Returns an array containing the property definitions specified in the
     * declaration of <i>this</i> node type. This does <i>not</i> include
     * property definitions inherited from supertypes of this node type.
     *
     * @see #getPropertyDefs
     *
     * @return an array containing the property definitions.
     */
    public PropertyDef[] getDeclaredPropertyDefs();

    /**
     * Returns an array containing the child node definitions specified in the
     * declaration of <i>this</i> node type. This does <i>not</i> include
     * child node definitions inherited from supertypes of this node type.
     *
     * @see #getChildNodeDefs
     *
     * @return an array containing the child node definitions.
     */
    public NodeDef[] getDeclaredChildNodeDefs();

    /**
     * Checks if setting <code>propertyName</code> to <code>value</code> is
     * allowed by this node type. If it is not allowed a
     * <code>ConstraintViolationException</code> is thrown.
     *
     * @param propertyName The name of the property
     * @param value A <code>Value</code> object.
     *
     * @throws ValueFormatException If the type or format of the specified value
     * is incompatible with the type of the specified property.
     *
     * @throws ConstraintViolationException If setting <code>propertyName</code>
     * to <code>value</code> would violate a constraint as defined by this
     * node type.
     *
     * @throws RepositoryException If another error occurs.
     */
    public void checkSetProperty(String propertyName, Value value) throws ValueFormatException, ConstraintViolationException, RepositoryException;

    /**
     * Checks if adding a child node called <code>childNodeName</code> of
     * node type <code>nodeTypeName</code> is allowed by <i>this</i> node type.
     * The same as <code>{@link #checkAddChildNode(String childNodeName)}</code>
     * except that the type of the child node is explictly specified.
     * <p>
     * If it is not allowed a <code>ConstraintViolationException</code> is
     * thrown.
     *
     * @param childNodeName The name of the child node.
     * @param nodeTypeName The name of the node type of the child node.
     *
     * @throws NoSuchNodeTypeException If the specified <code>nodeTypeName</code>
     * is not recognized.
     *
     * @throws ConstraintViolationException If adding a child node called
     * <code>childNodeName</code> of node type <code>nodeTypeName</code> would
     * violate a constraint as defined by <i>this</i> node type.
     *
     * @throws RepositoryException If another error occurs.
     */
    public void checkAddChildNode(String childNodeName, String nodeTypeName) throws NoSuchNodeTypeException, ConstraintViolationException, RepositoryException;

    /**
     * Checks if adding a child node called <code>childNodeName</code> is
     * allowed by <i>this</i> node type.
     * <p>
     * If it is not allowed or if the node type of the child node can not be
     * unambiguously determined a <code>ConstraintViolationException</code> is
     * thrown.
     *
     * @param childNodeName The name of the child node.
     *
     * @throws ConstraintViolationException If adding a child node called
     * <code>childNodeName</code> would violate a constraint as defined by
     * <i>this</i> node type or if the node type of the child node can not be
     * unambiguously determined.
     *
     * @throws RepositoryException If another error occurs.
     */
    public void checkAddChildNode(String childNodeName) throws ConstraintViolationException, RepositoryException;

    /**
     * Checks if removing <code>itemName</code> is allowed by this node type.
     * If it is not allowed a <code>ConstraintViolationException</code> is
     * thrown.
     *
     * @param itemName The name of the child item
     *
     * @throws ConstraintViolationException If removing <code>itemName</code>
     * would violate a constraint as defined by this node type.
     *
     * @throws RepositoryException If another error occurs.
     */
    public void checkRemoveItem(String itemName) throws ConstraintViolationException, RepositoryException;
}