/*
 * Query.java
 *
 * This file is part of the v0.12 specification for the
 * Java Content Repository API
 */
package javax.jcr.query;

import javax.jcr.ItemExistsException;
import javax.jcr.PathNotFoundException;
import javax.jcr.nodetype.ConstraintViolationException;
import javax.jcr.RepositoryException;

/**
 * A <code>Query</code> object.
 *
 * @author Peeter Piegaze
 */
public interface Query {

    /**
     * Executes this search and returns a
     * <code>{@link javax.jcr.query.QueryResultIterator QueryResultIterator}</code> object.
     */
    public QueryResultIterator execute();

    /**
     * Returns the statement set for this query. Returns <code>null</code>
     * if no statement is currently set.
     *
     * @return the query statement.
     */
    public String getStatement();

    /**
     * Returns the language defined for this query.
     * One of the strings returned by
     * <code>QueryManager.getSupportedQueryLanguages()</code>.
     * Returns <code>null</code> if no language is set.
     */
    public String getLanguage();

    /**
     * If this is a persistent query (i.e., it has been saved), returns the path of the
     * node that persists it. If this query has not been saved, it returns null.
     *
     * @return path of persisted node representing this query in content, or <code>null
     * </code> for a transient <code>Query</code>
     */
    public String getPersistentQueryPath();

    /**
     * Creates a persistent query. The persisted query will be created as a
     * node of node type <code>nt:query</code> upon success.
     *
     * @param absPath path at which to persist this query.
     * 
     * @throws an <code>ItemExistsException</code> if a node already exists at
     * <code>absPath</code>.
     * @throws a <code>PathNotFoundException</code> if <code>absPath</code> is not 
     * found.
     * @throws an <code>ConstraintViolationException</code> if creating the node 
     * would violate a node type (or other implementation specific) constraint.
     * @throws <code>RepositoryException</code> if other errors occur.
     * 
     */
    public void save(String absPath) throws ItemExistsException, PathNotFoundException, ConstraintViolationException, RepositoryException;
}
